# -*- coding: utf-8 -*-
from irekua_models.models import Model

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.object_types import annotations
from irekua_rest_api.serializers.terms import terms


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = Model
        fields = ('url', 'id', 'name')


class ListSerializer(IrekuaModelSerializer):
    class Meta:
        model = Model
        fields = (
            'url',
            'id',
            'name',
            'description'
        )


class DetailSerializer(IrekuaModelSerializer):
    annotation_type = annotations.SelectSerializer(read_only=True)
    terms = terms.ListSerializer(read_only=True, many=True)

    class Meta:
        model = Model
        fields = (
            'url',
            'id',
            'name',
            'description',
            'repository',
            'annotation_type',
            'terms',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = Model
        fields = (
            'name',
            'description',
            'repository',
            'annotation_type',
            'terms',
            'event_types',
            'item_types',
        )

        extra_kwargs = {
            'terms': {'style': {'base_template': 'input.html'}},
        }

    def create(self, validated_data):
        user = self.context['request'].user
        validated_data['created_by'] = user
        validated_data['modified_by'] = user
        return super().create(validated_data)


class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = Model
        fields = (
            'name',
            'description',
            'repository',
            'annotation_type',
            'terms',
            'event_types',
            'item_types',
        )

        extra_kwargs = {
            'terms': {'style': {'base_template': 'input.html'}},
        }

    def update(self, validated_data):
        user = self.context['request'].user
        validated_data['modified_by'] = user
        return super().create(validated_data)



import django_filters

from irekua_database.models import SamplingEvent
from .utils import BaseFilter


search_fields = (
    'sampling_event_type__name',
    'collection_site__site__site_type__name',
    'collection_site__site__locality',
)


class Filter(BaseFilter):
    latitude__gt = django_filters.NumberFilter(
        field_name='site.latitude',
        lookup_expr='gt')
    latitude__lt = django_filters.NumberFilter(
        field_name='site.latitude',
        lookup_expr='lt')

    longitude__gt = django_filters.NumberFilter(
        field_name='site.longitude',
        lookup_expr='gt')
    longitude__lt = django_filters.NumberFilter(
        field_name='site.longitude',
        lookup_expr='lt')

    altitude__gt = django_filters.NumberFilter(
        field_name='site.altitude',
        lookup_expr='gt')
    altitude__lt = django_filters.NumberFilter(
        field_name='site.altitude',
        lookup_expr='lt')

    started_on__gt = django_filters.NumberFilter(
        field_name='started_on',
        lookup_expr='gt')
    started_on__lt = django_filters.NumberFilter(
        field_name='started_on',
        lookup_expr='lt')

    ended_on__gt = django_filters.NumberFilter(
        field_name='ended_on',
        lookup_expr='gt')
    ended_on__lt = django_filters.NumberFilter(
        field_name='ended_on',
        lookup_expr='lt')

    class Meta:
        model = SamplingEvent
        fields = (
            'sampling_event_type__name',
            'collection_site__site_type',
            'collection_site__site__locality',
            'collection__collection_type__name',
            'collection__name',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import SamplingEvent

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from irekua_rest_api.serializers.data_collections import sites
from irekua_rest_api.serializers.users import users
from irekua_rest_api.serializers.object_types.sampling_events import types
from irekua_rest_api.serializers.data_collections import data_collections
from irekua_rest_api.serializers import licences


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = SamplingEvent
        fields = (
            'url',
            'id',
        )


class ListSerializer(IrekuaModelSerializer):
    collection_site = serializers.CharField(
        read_only=True,
        source='collection_site.site.name')
    site_internal_id = serializers.PrimaryKeyRelatedField(
        read_only=True,
        source='collection_site.internal_id')

    class Meta:
        model = SamplingEvent
        fields = (
            'url',
            'id',
            'sampling_event_type',
            'collection_site',
            'site_internal_id',
            'started_on',
            'ended_on',
        )


class UserListSerializer(IrekuaModelSerializer):
    collection_site = serializers.CharField(
        read_only=True,
        source='collection_site.site.name')
    site_internal_id = serializers.PrimaryKeyRelatedField(
        read_only=True,
        source='collection_site.internal_id')

    class Meta:
        model = SamplingEvent
        fields = (
            'url',
            'id',
            'collection',
            'sampling_event_type',
            'collection_site',
            'site_internal_id',
            'started_on',
            'ended_on',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    created_by = users.SelectSerializer(
        many=False,
        read_only=True)
    modified_by = users.SelectSerializer(
        many=False,
        read_only=True)
    sampling_event_type = types.SelectSerializer(
        many=False,
        read_only=True)
    collection = data_collections.SelectSerializer(
        many=False,
        read_only=True)
    licence = licences.SelectSerializer(
        many=False,
        read_only=True)
    collection_site = sites.SelectSerializer(
        many=False,
        read_only=True)

    class Meta:
        model = SamplingEvent
        fields = (
            'url',
            'id',
            'sampling_event_type',
            'collection_site',
            'commentaries',
            'metadata',
            'collection',
            'licence',
            'started_on',
            'ended_on',
            'created_by',
            'modified_by',
            'created_on',
            'modified_on',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = SamplingEvent
        fields = (
            'sampling_event_type',
            'collection_site',
            'commentaries',
            'metadata',
            'started_on',
            'ended_on',
            'licence',
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        try:
            collection = self.context['collection']

            sites_field = self.fields['collection_site']
            sites_field.queryset = collection.collectionsite_set.all()

            licences_field = self.fields['licence']
            licences_field.queryset = collection.licence_set.all()

        except (KeyError, AttributeError):
            pass

    def create(self, validated_data):
        user = self.context['request'].user
        collection = self.context['collection']

        validated_data['created_by'] = user
        validated_data['modified_by'] = user
        validated_data['collection'] = collection
        return super().create(validated_data)


class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = SamplingEvent
        fields = (
            'commentaries',
            'metadata',
            'started_on',
            'ended_on',
        )

    def update(self, validated_data):
        user = self.context['request'].user
        validated_data['modified_by'] = user
        return super().update(validated_data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import PhysicalDevice

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from irekua_rest_api.serializers.users import users
from . import devices


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = PhysicalDevice
        fields = (
            'url',
            'id',
        )


class ListSerializer(IrekuaModelSerializer):
    type = serializers.CharField(
        read_only=True,
        source='device.device_type.name')
    brand = serializers.CharField(
        read_only=True,
        source='device.brand.name')
    model = serializers.CharField(
        read_only=True,
        source='device.model')

    class Meta:
        model = PhysicalDevice
        fields = (
            'url',
            'id',
            'serial_number',
            'type',
            'brand',
            'model',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    device = devices.SelectSerializer(many=False, read_only=True)
    owner = users.SelectSerializer(many=False, read_only=True)

    class Meta:
        model = PhysicalDevice
        fields = (
            'url',
            'serial_number',
            'owner',
            'metadata',
            'bundle',
            'device',
            'created_on',
            'modified_on',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = PhysicalDevice
        fields = (
            'serial_number',
            'device',
            'metadata',
            'bundle',
        )

    def create(self, validated_data):
        user = self.context['request'].user
        validated_data['owner'] = user
        return super().create(validated_data)


class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = PhysicalDevice
        fields = (
            'serial_number',
            'metadata',
        )



from . import annotations
from . import devices
from . import entailments
from . import events
from . import items
from . import licences
from . import sites
from . import terms
from . import data_collections
from . import sampling_events
from . import mime_types


__all__ = [
    'annotations',
    'devices',
    'entailments',
    'events',
    'items',
    'licences',
    'sites',
    'terms',
    'data_collections',
    'sampling_events',
    'mime_types',
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from irekua_database.models import SamplingEventType

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = SamplingEventType
        fields = (
            'url',
            'name'
        )


class ListSerializer(IrekuaModelSerializer):
    class Meta:
        model = SamplingEventType
        fields = (
            'url',
            'name',
            'icon',
            'description',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    class Meta:
        model = SamplingEventType
        fields = (
            'url',
            'name',
            'description',
            'icon',
            'metadata_schema',
            'restrict_device_types',
            'restrict_site_types',
            'created_on',
            'modified_on',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = SamplingEventType
        fields = (
            'name',
            'description',
            'icon',
            'metadata_schema',
            'restrict_device_types',
            'restrict_site_types',
        )


class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = SamplingEventType
        fields = (
            'description',
            'icon',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import CollectionUser

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from irekua_rest_api.serializers.users import users
from irekua_rest_api.serializers.users import roles


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = CollectionUser
        fields = (
            'url',
            'id',
        )


class ListSerializer(IrekuaModelSerializer):
    user = serializers.SlugRelatedField(
        many=False,
        read_only=True,
        slug_field='username')
    role = serializers.SlugRelatedField(
        many=False,
        read_only=True,
        slug_field='name')

    class Meta:
        model = CollectionUser
        fields = (
            'url',
            'id',
            'user',
            'role',
            'metadata',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    user = users.SelectSerializer(many=False, read_only=True)
    role = roles.SelectSerializer(many=False, read_only=True)

    class Meta:
        model = CollectionUser
        fields = (
            'url',
            'user',
            'role',
            'metadata',
            'created_on',
            'modified_on',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = CollectionUser
        fields = (
            'user',
            'role',
            'metadata',
        )

    def create(self, validated_data):
        collection = self.context['collection']
        validated_data['collection'] = collection
        return super().create(validated_data)


class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = CollectionUser
        fields = (
            'metadata',
        )


class RoleSerializer(IrekuaModelSerializer):
    class Meta:
        model = CollectionUser
        fields = (
            'role',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import mixins
from rest_framework.viewsets import GenericViewSet

from irekua_database import models
from irekua_rest_api import utils
from irekua_rest_api import serializers

from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import IsDeveloper
from irekua_rest_api.permissions import ReadOnly


class EntailmentTypeViewSet(mixins.RetrieveModelMixin,
                            mixins.DestroyModelMixin,
                            mixins.UpdateModelMixin,
                            utils.CustomViewSetMixin,
                            GenericViewSet):
    queryset = models.EntailmentType.objects.all()  # pylint: disable=E1101

    serializer_mapping = utils.SerializerMapping.from_module(
        serializers.object_types.entailments)

    permission_mapping = utils.PermissionMapping(
        default=IsDeveloper | IsAdmin | ReadOnly)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import mixins
from rest_framework.viewsets import GenericViewSet

from irekua_database import models
from irekua_rest_api import serializers
from irekua_rest_api import utils

from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import IsCurator
from irekua_rest_api.permissions import ReadOnly


class EntailmentViewSet(mixins.UpdateModelMixin,
                        mixins.RetrieveModelMixin,
                        mixins.DestroyModelMixin,
                        utils.CustomViewSetMixin,
                        GenericViewSet):
    queryset = models.Entailment.objects.all()  # pylint: disable=E1101

    serializer_mapping = utils.SerializerMapping.from_module(
        serializers.terms.entailments)

    permission_mapping = utils.PermissionMapping(
        default=IsAdmin | IsCurator | ReadOnly)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import CollectionItemType

from irekua_rest_api.serializers.object_types import items

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from irekua_rest_api.serializers.object_types import events
from irekua_rest_api.serializers.object_types import mime_types
from . import types


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = CollectionItemType
        fields = (
            'url',
            'id',
        )


class ListSerializer(IrekuaModelSerializer):
    event_types = events.SelectSerializer(
        many=True,
        read_only=True,
        source='item_type.event_types')
    mime_types = mime_types.SelectSerializer(
        many=True,
        read_only=True,
        source='item_type.mime_types')
    name = serializers.SlugRelatedField(
        read_only=True,
        slug_field='name',
        source='item_type')

    class Meta:
        model = CollectionItemType
        fields = (
            'url',
            'id',
            'name',
            'item_type',
            'event_types',
            'mime_types',
            'metadata_schema',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    item_type = items.SelectSerializer(
        many=False,
        read_only=True)
    collection_type = types.SelectSerializer(
        many=False,
        read_only=True)
    event_types = events.SelectSerializer(
        many=True,
        read_only=True,
        source='item_type.event_types')
    mime_types = mime_types.SelectSerializer(
        many=True,
        read_only=True,
        source='item_type.mime_types')

    class Meta:
        model = CollectionItemType
        fields = (
            'url',
            'id',
            'collection_type',
            'item_type',
            'metadata_schema',
            'event_types',
            'mime_types',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = CollectionItemType
        fields = (
            'item_type',
            'metadata_schema',
        )

    def create(self, validated_data):
        collection_type = self.context['collection_type']
        validated_data['collection_type'] = collection_type
        return super().create(validated_data)


class SimpleListSerializer(IrekuaModelSerializer):
    event_types = events.DescriptionSerializer(
        many=True,
        read_only=True,
        source='item_type.event_types')
    mime_types = mime_types.DescriptionSerializer(
        many=True,
        read_only=True,
        source='item_type.mime_types')


    class Meta:
        model = CollectionItemType
        fields = (
            'id',
            'item_type',
            'event_types',
            'mime_types',
            'metadata_schema',
        )



from . import entailments
from . import synonym_suggestions
from . import synonyms
from . import suggestions
from . import terms


__all__ = [
    'entailments',
    'synonym_suggestions',
    'synonyms',
    'suggestions',
    'terms',
]



from django import forms
import django_filters

from irekua_database.models import Term
from irekua_models.models import ModelVersion
from .utils import BaseFilter


search_fields = (
    'version',
    'model__name',
    'model__annotation_type__name',
    'model__event_types__name',
    'model__terms__value',
)


class Filter(BaseFilter):
    terms = django_filters.ModelMultipleChoiceFilter(
        field_name='model__terms',
        queryset=Term.objects.all(),
        widget=forms.TextInput)

    class Meta:
        model = ModelVersion
        fields = {
            'version': ['exact', 'icontains', 'lt', 'gt', 'lte', 'gte'],
            'model': ['exact'],
            'model__name': ['exact', 'icontains'],
            'model__annotation_type': ['exact'],
            'model__annotation_type__name': ['exact', 'icontains'],
            'model__event_types': ['exact'],
            'model__event_types__name': ['exact', 'icontains'],
            'model__terms__value': ['exact', 'icontains']
        }



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import mixins
from rest_framework.viewsets import GenericViewSet

from irekua_database import models
from irekua_rest_api import utils
from irekua_rest_api import serializers

from irekua_rest_api.permissions import IsAuthenticated
from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import IsSpecialUser
from irekua_rest_api.permissions import collection_sites as permissions


class CollectionSiteViewSet(mixins.UpdateModelMixin,
                            mixins.RetrieveModelMixin,
                            mixins.DestroyModelMixin,
                            utils.CustomViewSetMixin,
                            GenericViewSet):
    queryset = models.CollectionSite.objects.all()  # pylint: disable=E1101

    serializer_mapping = utils.SerializerMapping.from_module(
        serializers.data_collections.sites)

    permission_mapping = utils.PermissionMapping({
        utils.Actions.UPDATE: [
            IsAuthenticated,
            (
                permissions.IsCreator |
                permissions.HasUpdatePermission |
                IsAdmin
            ),
        ],
        utils.Actions.RETRIEVE: [
            IsAuthenticated,
            (
                permissions.IsCreator |
                permissions.HasViewPermission |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                IsSpecialUser
            ),
        ],
        utils.Actions.DESTROY: [
            IsAuthenticated,
            (
                permissions.IsCreator |
                IsAdmin
            ),
        ]
    })



from uuid import uuid4
from random import sample

from rest_framework.test import APITestCase
from irekua_database.utils import simple_JSON_schema
from irekua_database.models import TermType
from irekua_rest_api.serializers import entailment_types
from .utils import (
    BaseTestCase,
    Users,
    Actions,
    create_permission_mapping_from_lists
)


class EntailmentTypeTestCase(BaseTestCase, APITestCase):
    serializer = entailment_types.CreateSerializer
    permissions = create_permission_mapping_from_lists({
        Actions.LIST: Users.ALL_AUTHENTICATED_USERS,
        Actions.CREATE: [
            Users.ADMIN,
            Users.DEVELOPER],
        Actions.RETRIEVE: Users.ALL_AUTHENTICATED_USERS,
        Actions.UPDATE: [
            Users.ADMIN,
            Users.DEVELOPER],
        Actions.PARTIAL_UPDATE: [
            Users.ADMIN,
            Users.DEVELOPER],
        Actions.DESTROY: [
            Users.ADMIN,
            Users.DEVELOPER],
    })

    term_type_names = [str(uuid4()) for _ in range(50)]

    def setUp(self):
        super().setUp()

        for term_type_name in self.term_type_names:
            TermType.objects.create(
                name=term_type_name,
                description='random term type',
                is_categorical=True)

    @staticmethod
    def generate_random_json_data():
        source_type, target_type = sample(
            EntailmentTypeTestCase.term_type_names, 2)
        data = {
            'source_type': source_type,
            'target_type': target_type,
            'metadata_schema': simple_JSON_schema()
        }
        return data



from rest_framework.schemas import AutoSchema


class CustomSchema(AutoSchema):
    def _allows_filters(self, path, method):
        if method.lower() != 'get':
            return False

        if getattr(self.view, 'filter_backends', None) is None:
            return False

        if hasattr(self.view, 'action'):
            if self.view.action in ["retrieve", "update", "partial_update", "destroy"]:
                return False

            if self.view.action == 'list':
                return True

            return self.view.action[-1] == 's'

        return False



from rest_framework.serializers import ModelSerializer
from rest_framework.serializers import HyperlinkedModelSerializer
from irekua_rest_api.utils.urls import get_detail_view_name


class IrekuaModelSerializer(ModelSerializer):
    def build_url_field(self, field_name, model_class):
        field_class = self.serializer_url_field
        field_kwargs = {'view_name': get_detail_view_name(model_class)}
        return field_class, field_kwargs


class IrekuaHyperlinkedModelSerializer(HyperlinkedModelSerializer):
    def build_url_field(self, field_name, model_class):
        field_class = self.serializer_url_field
        field_kwargs = {'view_name': get_detail_view_name(model_class)}
        return field_class, field_kwargs



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import User

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from . import institutions


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = User
        fields = (
            'url',
            'id',
            'username',
        )


class ListSerializer(IrekuaModelSerializer):
    institution = serializers.SlugRelatedField(
        many=False,
        read_only=True,
        slug_field='institution_name')

    class Meta:
        model = User
        fields = (
            'url',
            'id',
            'username',
            'institution'
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    institution = institutions.SelectSerializer(many=False, read_only=True)

    class Meta:
        model = User
        fields = (
            'url',
            'id',
            'username',
            'first_name',
            'last_name',
            'institution',
        )


class FullDetailSerializer(IrekuaHyperlinkedModelSerializer):
    institution = institutions.DetailSerializer(many=False, read_only=True)

    class Meta:
        model = User
        fields = (
            'url',
            'id',
            'username',
            'email',
            'first_name',
            'last_name',
            'institution',
            'last_login',
            'date_joined',
            'is_superuser',
            'is_curator',
            'is_developer',
            'is_model',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = User
        fields = (
            'username',
            'email',
            'password',
            'first_name',
            'last_name',
            'institution',
        )


class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = User
        fields = (
            'username',
            'email',
            'first_name',
            'last_name',
            'institution',
        )



from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from rest_framework.permissions import SAFE_METHODS


class ReadOnly(IsAuthenticated):
    def has_permission(self, request, view):
        is_auth = super().has_permission(request, view)
        if not is_auth:
            return False

        return request.method in SAFE_METHODS


class IsUnauthenticated(IsAuthenticated):
    def has_permission(self, request, view):
        return not super().has_permission(request, view)


class IsDeveloper(BasePermission):
    def has_permission(self, request, view):
        user = request.user
        return user.is_developer


class IsModel(BasePermission):
    def has_permission(self, request, view):
        user = request.user
        return user.is_model


class IsCurator(BasePermission):
    def has_permission(self, request, view):
        user = request.user
        return user.is_curator


class IsAdmin(BasePermission):
    def has_permission(self, request, view):
        user = request.user
        return user.is_superuser


class IsSpecialUser(BasePermission):
    def has_permission(self, request, view):
        user = request.user
        return user.is_superuser | user.is_curator | user.is_model | user.is_developer



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from irekua_database.models import SamplingEventTypeDeviceType

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from irekua_rest_api.serializers.object_types import devices
from . import types


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = SamplingEventTypeDeviceType
        fields = (
            'url',
            'id'
        )


class ListSerializer(IrekuaModelSerializer):
    class Meta:
        model = SamplingEventTypeDeviceType
        fields = (
            'url',
            'id',
            'sampling_event_type',
            'device_type',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    device_type = devices.SelectSerializer(
        many=False,
        read_only=True)
    sampling_event_type = types.SelectSerializer(
        many=False,
        read_only=True)

    class Meta:
        model = SamplingEventTypeDeviceType
        fields = (
            'url',
            'id',
            'sampling_event_type',
            'device_type',
            'metadata_schema',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = SamplingEventTypeDeviceType
        fields = (
            'device_type',
            'metadata_schema',
        )

    def create(self, validated_data):
        sampling_event_type = self.context['sampling_event_type']
        validated_data['sampling_event_type'] = sampling_event_type
        return super().create(validated_data)


class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = SamplingEventTypeDeviceType
        fields = (
            'metadata_schema',
        )



from rest_framework.permissions import BasePermission


class IsOwnSuggestion(BasePermission):
    def has_permission(self, request, view):
        try:
            suggestion = view.get_object()
            user = request.user
            return suggestion.suggested_by == user
        except (AttributeError, AssertionError):
            return False



# pylint: disable=C0301
from .annotations.annotation_tools import AnnotationToolViewSet
from .annotations.annotation_votes import AnnotationVoteViewSet
from .annotations.annotations import AnnotationViewSet
from .data_collections.collection_devices import CollectionDeviceViewSet
from .data_collections.collection_sites import CollectionSiteViewSet
from .data_collections.collection_users import CollectionUserViewSet
from .data_collections.data_collections import CollectionViewSet
from .data_collections.metacollections import MetaCollectionViewSet
from .data_collections.administrators import CollectionAdministratorViewSet
from .devices.device_brands import DeviceBrandViewSet
from .devices.devices import DeviceViewSet
from .devices.physical_devices import PhysicalDeviceViewSet
from .items.items import ItemViewSet
from .licences import LicenceViewSet
from .object_types.annotation_types import AnnotationTypeViewSet
from .object_types.data_collections.collection_administrators import CollectionTypeAdministratorViewSet
from .object_types.data_collections.collection_annotation_types import CollectionTypeAnnotationTypeViewSet
from .object_types.data_collections.collection_licence_types import CollectionTypeLicenceTypeViewSet
from .object_types.data_collections.collection_sampling_event_types import CollectionTypeSamplingEventTypeViewSet
from .object_types.data_collections.collection_site_types import CollectionTypeSiteTypeViewSet
from .object_types.data_collections.collection_event_types import CollectionTypeEventTypeViewSet
from .object_types.data_collections.collection_types import CollectionTypeViewSet
from .object_types.data_collections.collection_item_types import CollectionTypeItemTypeViewSet
from .object_types.data_collections.collection_device_types import CollectionTypeDeviceTypeViewSet
from .object_types.data_collections.collection_roles import CollectionTypeRoleViewSet
from .object_types.device_types import DeviceTypeViewSet
from .object_types.entailment_types import EntailmentTypeViewSet
from .object_types.event_types import EventTypeViewSet
from .object_types.item_types import ItemTypeViewSet
from .object_types.mime_types import MimeTypeViewSet
from .object_types.licence_types import LicenceTypeViewSet
from .object_types.sampling_events.sampling_event_type_device_types import SamplingEventTypeDeviceTypeViewSet
from .object_types.sampling_events.sampling_event_type_site_types import SamplingEventTypeSiteTypeViewSet
from .object_types.sampling_events.sampling_event_types import SamplingEventTypeViewSet
from .object_types.site_types import SiteTypeViewSet
from .object_types.term_types import TermTypeViewSet
from .sampling_events.sampling_event_devices import SamplingEventDeviceViewSet
from .sampling_events.sampling_events import SamplingEventViewSet
from .items.secondary_items import SecondaryItemViewSet
from .sites import SiteViewSet
from .items.tags import TagViewSet
from .terms.entailments import EntailmentViewSet
from .terms.synonym_suggestions import SynonymSuggestionViewSet
from .terms.synonyms import SynonymViewSet
from .terms.term_suggestions import TermSuggestionViewSet
from .terms.terms import TermViewSet
from .users.institutions import InstitutionViewSet
from .users.roles import RoleViewSet
from .users.users import UserViewSet
from .models.model import ModelViewSet
from .models.model_version import ModelVersionViewSet
from .models.model_prediction import ModelPredictionViewSet


__all__ = [
    'AnnotationToolViewSet',
    'AnnotationTypeViewSet',
    'AnnotationViewSet',
    'AnnotationVoteViewSet',
    'CollectionDeviceViewSet',
    'CollectionSiteViewSet',
    'CollectionTypeAdministratorViewSet',
    'CollectionTypeAnnotationTypeViewSet',
    'CollectionTypeLicenceTypeViewSet',
    'CollectionTypeSamplingEventTypeViewSet',
    'CollectionTypeItemTypeViewSet',
    'CollectionTypeSiteTypeViewSet',
    'CollectionTypeEventTypeViewSet',
    'CollectionTypeViewSet',
    'CollectionUserViewSet',
    'CollectionViewSet',
    'DeviceBrandViewSet',
    'DeviceTypeViewSet',
    'DeviceViewSet',
    'EntailmentTypeViewSet',
    'EntailmentViewSet',
    'EventTypeViewSet',
    'InstitutionViewSet',
    'ItemTypeViewSet',
    'ItemViewSet',
    'LicenceTypeViewSet',
    'LicenceViewSet',
    'MetaCollectionViewSet',
    'PhysicalDeviceViewSet',
    'RoleViewSet',
    'SamplingEventDeviceViewSet',
    'SamplingEventTypeDeviceTypeViewSet',
    'SamplingEventTypeSiteTypeViewSet',
    'SamplingEventTypeViewSet',
    'SamplingEventViewSet',
    'SecondaryItemViewSet',
    'SiteTypeViewSet',
    'SiteViewSet',
    'SynonymSuggestionViewSet',
    'SynonymViewSet',
    'TagViewSet',
    'TermSuggestionViewSet',
    'TermTypeViewSet',
    'TermViewSet',
    'UserViewSet',
    'CollectionTypeDeviceTypeViewSet',
    'CollectionTypeRoleViewSet',
    'CollectionAdministratorViewSet',
    'MimeTypeViewSet',
    'ModelViewSet',
    'ModelVersionViewSet',
    'ModelPredictionViewSet'
]



from irekua_models.models import ModelPrediction
from .utils import BaseFilter


search_fields = (
    'model_version__model__name',
    'model_version__model__annotation_type__name',
    'model_version__version',
    'item__item_type__name',
    'event_type__name',
    'labels__value',
    'labels__term_type__name',
)


class Filter(BaseFilter):
    class Meta:
        model = ModelPrediction
        fields = {
            'item': ['exact'],
            'item__item_type': ['exact'],
            'item__item_type__name': ['exact', 'icontains'],
            'certainty': ['exact', 'lt', 'lte', 'gt', 'gte'],
            'model_version__model': ['exact'],
            'model_version__version': ['exact', 'icontains', 'gt', 'gte', 'lt', 'lte'],
            'model_version__model__name': ['exact', 'icontains'],
            'model_version__model__annotation_type': ['exact'],
            'model_version__model__annotation_type__name': ['exact', 'icontains'],
            'event_type': ['exact'],
            'event_type__name': ['exact', 'icontains'],
            'labels': ['exact'],
            'labels__value': ['exact', 'icontains'],
            'labels__term_type': ['exact'],
            'labels__term_type__name': ['exact', 'icontains']
        }



from uuid import uuid4

from rest_framework.test import APITestCase

from irekua_database.models import TermType
from irekua_database.utils import simple_JSON_schema
from irekua_rest_api.serializers import event_types

from .utils import (
    BaseTestCase,
    Users,
    Actions,
    create_permission_mapping_from_lists
)


class EventTypeTestCase(BaseTestCase, APITestCase):
    serializer = event_types.CreateSerializer
    permissions = create_permission_mapping_from_lists({
        Actions.LIST: Users.ALL_AUTHENTICATED_USERS,
        Actions.CREATE: [
            Users.ADMIN],
        Actions.RETRIEVE: Users.ALL_AUTHENTICATED_USERS,
        Actions.UPDATE: [
            Users.ADMIN],
        Actions.PARTIAL_UPDATE: [
            Users.ADMIN],
        Actions.DESTROY: [
            Users.ADMIN],
    })

    @staticmethod
    def generate_random_json_data():
        data = {
            'name': str(uuid4()),
            'description': 'Random event type',
        }
        return data

    def setUp(self):
        super(EventTypeTestCase, self).setUp()

        self.event_type = self.create_random_object()
        self.term_type = TermType.objects.create(
            name=str(uuid4()),
            description='random term type',
            is_categorical=True,
            metadata_schema=simple_JSON_schema(),
            synonym_metadata_schema=simple_JSON_schema())



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.shortcuts import get_object_or_404
from rest_framework import mixins
from rest_framework.decorators import action
from rest_framework.viewsets import GenericViewSet

from irekua_database import models
from irekua_rest_api import serializers
from irekua_rest_api import utils


class CollectionTypeViewSet(mixins.RetrieveModelMixin,
                            mixins.DestroyModelMixin,
                            mixins.UpdateModelMixin,
                            utils.CustomViewSetMixin,
                            GenericViewSet):
    queryset = models.CollectionType.objects.all()  # pylint: disable=E1101

    permission_mapping = utils.PermissionMapping()

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.object_types.data_collections.types)
        .extend(
            administrators=serializers.object_types.data_collections.administrators.ListSerializer,  # noqa pylint: disable=C0301
            add_administrator=serializers.object_types.data_collections.administrators.CreateSerializer,  # noqa pylint: disable=C0301
            site_types=serializers.object_types.data_collections.sites.ListSerializer,  # noqa pylint: disable=C0301
            add_site_type=serializers.object_types.data_collections.sites.CreateSerializer,  # noqa pylint: disable=C0301
            annotation_types=serializers.object_types.data_collections.annotations.ListSerializer,  # noqa pylint: disable=C0301
            add_annotation_type=serializers.object_types.data_collections.annotations.CreateSerializer,  # noqa pylint: disable=C0301
            licence_types=serializers.object_types.data_collections.licences.ListSerializer,  # noqa pylint: disable=C0301
            add_licence_type=serializers.object_types.data_collections.licences.CreateSerializer,  # noqa pylint: disable=C0301
            sampling_event_types=serializers.object_types.data_collections.sampling_events.ListSerializer,  # noqa pylint: disable=C0301
            add_sampling_event_type=serializers.object_types.data_collections.sampling_events.CreateSerializer,  # noqa pylint: disable=C0301
            item_types=serializers.object_types.data_collections.items.ListSerializer,  # noqa pylint: disable=C0301
            add_item_type=serializers.object_types.data_collections.items.CreateSerializer,  # noqa pylint: disable=C0301
            event_types=serializers.object_types.data_collections.events.ListSerializer,  # noqa pylint: disable=C0301
            add_event_type=serializers.object_types.data_collections.events.CreateSerializer,  # noqa pylint: disable=C0301
            device_types=serializers.object_types.data_collections.devices.ListSerializer,  # noqa pylint: disable=C0301
            add_device_type=serializers.object_types.data_collections.devices.CreateSerializer,  # noqa pylint: disable=C0301
            roles=serializers.object_types.data_collections.roles.ListSerializer,  # noqa pylint: disable=C0301
            add_role=serializers.object_types.data_collections.roles.CreateSerializer,  # noqa pylint: disable=C0301
        ))

    def get_object(self):
        type_pk = self.kwargs['pk']
        type = get_object_or_404(models.CollectionType, pk=type_pk)

        self.check_object_permissions(self.request, type)
        return type

    def get_serializer_context(self):
        context = super().get_serializer_context()

        try:
            collection_type = self.get_object()
        except (KeyError, AttributeError, AssertionError):
            collection_type = None

        context['collection_type'] = collection_type
        return context

    def get_queryset(self):
        if self.action == 'site_types':
            model = models.CollectionType.site_types.through  # pylint: disable=E1101
            collection_id = self.kwargs['pk']
            return model.objects.filter(collectiontype_id=collection_id)

        if self.action == 'administrators':
            model = models.CollectionType.administrators.through  # pylint: disable=E1101
            collection_id = self.kwargs['pk']
            return model.objects.filter(collectiontype_id=collection_id)

        if self.action == 'annotation_types':
            model = models.CollectionType.annotation_types.through  # pylint: disable=E1101
            collection_id = self.kwargs['pk']
            return model.objects.filter(collectiontype_id=collection_id)

        if self.action == 'licence_types':
            model = models.CollectionType.licence_types.through  # pylint: disable=E1101
            collection_id = self.kwargs['pk']
            return model.objects.filter(collectiontype_id=collection_id)

        if self.action == 'sampling_event_types':
            model = models.CollectionType.sampling_event_types.through  # pylint: disable=E1101
            collection_id = self.kwargs['pk']
            return model.objects.filter(collectiontype_id=collection_id)

        if self.action == 'item_types':
            collection_id = self.kwargs['pk']
            return models.CollectionItemType.objects.filter(  # pylint: disable=E1101
                collection_type=collection_id)

        if self.action == 'event_types':
            model = models.CollectionType.event_types.through  # pylint: disable=E1101
            collection_id = self.kwargs['pk']
            return model.objects.filter(collectiontype_id=collection_id)

        if self.action == 'device_types':
            collection_id = self.kwargs['pk']
            return models.CollectionDeviceType.objects.filter(  # pylint: disable=E1101
                collection_type=collection_id)

        if self.action == 'roles':
            collection_id = self.kwargs['pk']
            return models.CollectionRole.objects.filter(  # pylint: disable=E1101
                collection_type=collection_id)

        return super().get_queryset()

    @action(detail=True, methods=['GET'])
    def site_types(self, request, pk=None):
        return self.list_related_object_view()

    @site_types.mapping.post
    def add_site_type(self, request, pk=None):
        return self.create_related_object_view()

    @action(detail=True, methods=['GET'])
    def administrators(self, request, pk=None):
        return self.list_related_object_view()

    @administrators.mapping.post
    def add_administrator(self, request, pk=None):
        return self.create_related_object_view()

    @action(detail=True, methods=['GET'])
    def annotation_types(self, request, pk=None):
        return self.list_related_object_view()

    @annotation_types.mapping.post
    def add_annotation_type(self, request, pk=None):
        return self.create_related_object_view()

    @action(detail=True, methods=['GET'])
    def licence_types(self, request, pk=None):
        return self.list_related_object_view()

    @licence_types.mapping.post
    def add_licence_type(self, request, pk=None):
        return self.create_related_object_view()

    @action(detail=True, methods=['GET'])
    def sampling_event_types(self, request, pk=None):
        return self.list_related_object_view()

    @sampling_event_types.mapping.post
    def add_sampling_event_type(self, request, pk=None):
        return self.create_related_object_view()

    @action(detail=True, methods=['GET'])
    def item_types(self, request, pk=None):
        return self.list_related_object_view()

    @item_types.mapping.post
    def add_item_type(self, request, pk=None):
        return self.create_related_object_view()

    @action(detail=True, methods=['GET'])
    def event_types(self, request, pk=None):
        return self.list_related_object_view()

    @event_types.mapping.post
    def add_event_type(self, request, pk=None):
        return self.create_related_object_view()

    @action(detail=True, methods=['GET'])
    def device_types(self, request, pk=None):
        return self.list_related_object_view()

    @device_types.mapping.post
    def add_device_type(self, request, pk=None):
        return self.create_related_object_view()

    @action(detail=True, methods=['GET'])
    def roles(self, request, pk=None):
        return self.list_related_object_view()

    @roles.mapping.post
    def add_role(self, request, pk=None):
        return self.create_related_object_view()



from irekua_database.models import SamplingEventDevice
from .utils import BaseFilter


search_fields = (
    'collection_device__physical_device__device__brand__name',
    'collection_device__physical_device__device__model',
    'collection_device__physical_device__device__device_type__name',
)


class Filter(BaseFilter):
    class Meta:
        model = SamplingEventDevice
        fields = (
            'collection_device__physical_device__device__brand__name',
            'collection_device__physical_device__device__model',
            'collection_device__physical_device__device__device_type__name',
            'collection_device__physical_device__created_by__username',
            'collection_device__physical_device__created_by__first_name',
            'created_by__username',
            'created_by__first_name',
            'created_by__last_name',
            'modified_by__username',
            'modified_by__first_name',
            'modified_by__last_name',
            'licence__licence_type__name',
            'licence__is_active',
            'licence__licence_type__can_view',
            'licence__licence_type__can_download',
            'licence__licence_type__can_view_annotations',
            'licence__licence_type__can_annotate',
            'licence__licence_type__can_vote_annotations',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from irekua_database.models import MimeType

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = MimeType
        fields = (
            'url',
            'mime_type',
            'media_info_schema'
        )


class DescriptionSerializer(IrekuaModelSerializer):
    class Meta:
        model = MimeType
        fields = (
            'mime_type',
            'media_info_schema'
        )


class ListSerializer(IrekuaModelSerializer):
    class Meta:
        model = MimeType
        fields = (
            'url',
            'mime_type',
            'media_info_schema',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    class Meta:
        model = MimeType
        fields = (
            'url',
            'mime_type',
            'media_info_schema',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = MimeType
        fields = (
            'mime_type',
            'media_info_schema',
        )


class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = MimeType
        fields = (
            'mime_type',
            'media_info_schema',
        )



from irekua_database.models import SecondaryItem
from .utils import BaseFilter


search_fields = (
    'item_type__name',
)


class Filter(BaseFilter):
    class Meta:
        model = SecondaryItem
        fields = (
            'item_type__name',
        )



from rest_framework.views import exception_handler
from rest_framework import serializers
# from rest_framework import status
# from rest_framework.response import Response
from django.core.exceptions import ValidationError


def custom_exception_handler(exc, context):
    if isinstance(exc, ValidationError):
        exc = serializers.ValidationError(exc.message_dict)

    response = exception_handler(exc, context)

    if response is not None:
        response.data['status_code'] = response.status_code
        return response



from rest_framework.permissions import BasePermission

from irekua_database.models import Collection


class HasUpdatePermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        if not isinstance(obj, Collection):
            return True

        user = request.user
        return obj.has_permission(user, 'change_collection')


class IsCollectionAdmin(BasePermission):
    def has_object_permission(self, request, view, obj):
        if not isinstance(obj, Collection):
            return True

        user = request.user
        return obj.is_admin(user)


class IsCollectionTypeAdmin(BasePermission):
    def has_object_permission(self, request, view, obj):
        if not isinstance(obj, Collection):
            return True

        user = request.user
        collection_type = obj.collection_type
        return collection_type.is_admin(user)

class IsCollectionUser(BasePermission):
    def has_object_permission(self, request, view, obj):
        if not isinstance(obj, Collection):
            return True

        user = request.user
        return obj.has_user(user)


class HasAddLicencePermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        if not isinstance(obj, Collection):
            return True

        user = request.user
        return obj.has_permission(user, 'add_collection_licence')


class HasAddDevicePermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        if not isinstance(obj, Collection):
            return True

        user = request.user
        return obj.has_permission(user, 'add_collection_device')


class HasAddSitePermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        if not isinstance(obj, Collection):
            return True

        user = request.user
        return obj.has_permission(user, 'add_collection_site')


class HasAddUserPermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        if not isinstance(obj, Collection):
            return True

        user = request.user
        return obj.has_permission(user, 'add_collection_user')


class HasAddItemPermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        if not isinstance(obj, Collection):
            return True

        user = request.user
        return obj.has_permission(user, 'add_collection_item')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.shortcuts import get_object_or_404
from rest_framework.viewsets import GenericViewSet
from rest_framework.decorators import action
from rest_framework import mixins
from rest_framework.response import Response

from irekua_database import models
from irekua_rest_api import serializers
from irekua_rest_api import filters
from irekua_rest_api import utils

from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import IsAuthenticated
from irekua_rest_api.permissions import IsUnauthenticated
from irekua_rest_api.permissions import users as permissions


class UserViewSet(mixins.ListModelMixin,
                  mixins.UpdateModelMixin,
                  mixins.CreateModelMixin,
                  mixins.RetrieveModelMixin,
                  utils.CustomViewSetMixin,
                  GenericViewSet):
    queryset = models.User.objects.all()  # pylint: disable=E1101
    filterset_class = filters.users.Filter
    search_fields = filters.users.search_fields

    permission_mapping = utils.PermissionMapping({
        utils.Actions.CREATE: IsUnauthenticated,
        utils.Actions.UPDATE: [
            IsAuthenticated,
            permissions.IsSelf | IsAdmin
        ], # TODO: Fix permissions
    }, default=IsAuthenticated)

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.users.users)
        .extend(
            items=serializers.items.items.ListSerializer,
            devices=serializers.devices.physical_devices.ListSerializer,
            sites=serializers.sites.ListSerializer,
            roles=serializers.users.roles.ListSerializer,
            sampling_events=serializers.sampling_events.sampling_events.UserListSerializer,
            add_role=serializers.users.roles.CreateSerializer,
            collections=serializers.data_collections.data_collections.ListSerializer,
            institutions=serializers.users.institutions.ListSerializer,
            add_institution=serializers.users.institutions.CreateSerializer,
        ))

    def get_object(self):
        user_pk = self.kwargs['pk']
        user = get_object_or_404(models.User, pk=user_pk)

        self.check_object_permissions(self.request, user)
        return user

    def get_serializer_class(self):
        if self.action == 'retrieve':
            try:
                user = self.request.user
                viewed_user = self.get_object()

                if user == viewed_user or user.is_superuser:
                    return serializers.users.users.FullDetailSerializer
            except (AssertionError, AttributeError):
                return serializers.users.users.DetailSerializer
        return super().get_serializer_class()

    def get_queryset(self):
        if self.action == 'roles':
            return models.Role.objects.all()  # pylint: disable=E1101

        if self.action == 'institutions':
            return models.Institution.objects.all()  # pylint: disable=E1101

        if self.action == 'items' or self.action == 'item_locations':
            user_id = self.kwargs['pk']
            return models.Item.objects.filter(created_by=user_id)  # pylint: disable=E1101

        if self.action == 'devices':
            user_id = self.kwargs['pk']
            return models.PhysicalDevice.objects.filter(owner=user_id)  # pylint: disable=E1101

        if self.action == 'sites' or self.action == 'site_locations':
            user_id = self.kwargs['pk']
            return models.Site.objects.filter(created_by=user_id)  # pylint: disable=E1101

        if self.action == 'collections':
            user = self.get_object()
            return user.collection_users.all()

        if self.action == 'sampling_events' or self.action == 'sampling_event_locations':
            user = self.get_object()
            return user.sampling_event_created_by.all()

        return super().get_queryset()

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.roles.Filter,
        search_fields=filters.roles.search_fields)
    def roles(self, request):
        return self.list_related_object_view()

    @roles.mapping.post
    def add_role(self, request):
        return self.create_related_object_view()

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.institutions.Filter,
        search_fields=filters.institutions.search_fields)
    def institutions(self, request):
        return self.list_related_object_view()

    @institutions.mapping.post
    def add_institution(self, request):
        return self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.items.Filter,
        search_fields=filters.items.search_fields)
    def items(self, request, pk=None):
        return self.list_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.physical_devices.Filter,
        search_fields=filters.physical_devices.search_fields)
    def devices(self, request, pk=None):
        return self.list_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.sites.Filter,
        search_fields=filters.sites.search_fields)
    def sites(self, request, pk=None):
        return self.list_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.sampling_events.Filter,
        search_fields=filters.sampling_events.search_fields)
    def sampling_events(self, request, pk=None):
        return self.list_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.data_collections.Filter,
        search_fields=filters.data_collections.search_fields)
    def collections(self, request, pk=None):
        return self.list_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.sites.Filter,
        search_fields=filters.sites.search_fields)
    def site_locations(self, request, pk=None):
        queryset = self.filter_queryset(self.get_queryset())

        serializer = serializers.sites.SiteLocationSerializer(
            queryset,
            many=True,
            read_only=True)
        return Response(serializer.data)

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.items.Filter,
        search_fields=filters.items.search_fields)
    def item_locations(self, request, pk=None):
        queryset = self.filter_queryset(self.get_queryset())

        serializer = serializers.sites.ItemLocationSerializer(
            queryset,
            many=True,
            read_only=True)
        return Response(serializer.data)

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.sampling_events.Filter,
        search_fields=filters.sampling_events.search_fields)
    def sampling_event_locations(self, request, pk=None):
        queryset = self.filter_queryset(self.get_queryset())

        serializer = serializers.sites.SamplingEventLocationSerializer(
            queryset,
            many=True,
            read_only=True)
        return Response(serializer.data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import mixins
from rest_framework.decorators import action
from rest_framework.viewsets import GenericViewSet

from irekua_database import models
from irekua_rest_api import serializers
from irekua_rest_api import utils

from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import IsDeveloper
from irekua_rest_api.permissions import IsAuthenticated


class ItemTypeViewSet(mixins.RetrieveModelMixin,
                      mixins.DestroyModelMixin,
                      mixins.UpdateModelMixin,
                      utils.CustomViewSetMixin,
                      GenericViewSet):
    queryset = models.ItemType.objects.all()  # pylint: disable=E1101

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.object_types.items)
        .extend(
            add_event_types=serializers.object_types.events.SelectSerializer,
            remove_event_types=serializers.object_types.events.SelectSerializer
        ))

    permission_mapping = utils.PermissionMapping({
        utils.Actions.RETRIEVE: IsAuthenticated,
        utils.Actions.DESTROY: IsAdmin,
    }, default=IsDeveloper | IsAdmin)

    @action(detail=True, methods=['POST'])
    def add_event_types(self, request, pk=None):
        return self.add_related_object_view(models.EventType, 'event_type')

    @action(detail=True, methods=['POST'])
    def remove_event_types(self, request, pk=None):
        return self.remove_related_object_view('event_type')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import mixins
from rest_framework.decorators import action
from rest_framework.viewsets import GenericViewSet

from irekua_database import models
from irekua_rest_api import filters
from irekua_rest_api import utils
from irekua_rest_api import serializers

from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import IsDeveloper
from irekua_rest_api.permissions import IsSpecialUser
from irekua_rest_api.permissions import IsAuthenticated


class MetaCollectionViewSet(mixins.UpdateModelMixin,
                            mixins.RetrieveModelMixin,
                            mixins.DestroyModelMixin,
                            utils.CustomViewSetMixin,
                            GenericViewSet):
    queryset = models.MetaCollection.objects.all()  # pylint: disable=E1101
    filterset_class = filters.metacollections.Filter
    search_fields = filters.metacollections.search_fields

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.data_collections.metacollections)
        .extend(
            items=serializers.items.items.ListSerializer,
            add_item=serializers.items.items.SelectSerializer,
            remove_item=serializers.items.items.SelectSerializer,
            add_curator=serializers.users.users.SelectSerializer,
            remove_curator=serializers.users.users.SelectSerializer,
        ))

    permission_mapping = utils.PermissionMapping({
        utils.Actions.LIST: [
            IsAuthenticated
        ],
        utils.Actions.RETRIEVE: [
            IsAuthenticated,
            IsSpecialUser
        ],
        'items': [
            IsAuthenticated,
            IsSpecialUser
        ],
        'add_curator': [
            IsAuthenticated, IsAdmin
        ],
        'remove_curator': [
            IsAuthenticated, IsAdmin
        ],
    }, default=[
        IsAuthenticated,
        IsDeveloper | IsAdmin
    ])

    def get_queryset(self):
        if self.action == 'items':
            metacollection_id = self.kwargs['pk']
            metacollection = models.MetaCollection.objects.get(pk=metacollection_id)  # pylint: disable=E1101
            return metacollection.items.all()

        return super().get_queryset()

    @action(detail=True, methods=['POST'])
    def add_item(self, request, pk=None):
        return self.add_related_object_view(models.Item, 'item')

    @action(detail=True, methods=['POST'])
    def remove_item(self, request, pk=None):
        return self.remove_related_object_view('item')

    @action(detail=True, methods=['POST'])
    def add_curator(self, request, pk=None):
        return self.add_related_object_view(models.User, 'curator')

    @action(detail=True, methods=['POST'])
    def remove_curator(self, request, pk=None):
        return self.remove_related_object_view('curator')

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.items.Filter,
        search_fields=filters.items.search_fields)
    def items(self, request, pk=None):
        return self.list_related_object_view()



from rest_framework.permissions import BasePermission


class IsCreator(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        return obj.created_by == user


class IsCollectionAdmin(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        item = obj.item
        sampling_event = item.sampling_event_device.sampling_event
        collection = sampling_event.collection

        return collection.is_admin(user)


class IsCollectionTypeAdmin(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        item = obj.item
        sampling_event = item.sampling_event_device.sampling_event
        collection = sampling_event.collection
        collection_type = collection.collection_type

        return collection_type.is_admin(user)


class HasUpdatePermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        item = obj.item
        sampling_event = item.sampling_event_device.sampling_event
        collection = sampling_event.collection

        return collection.has_permission(
            user, 'change_collection_annotations')


class HasViewPermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        item = obj.item
        sampling_event = item.sampling_event_device.sampling_event
        collection = sampling_event.collection

        return collection.has_permission(
            user, 'view_collection_annotations')


class HasVotePermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        item = obj.item
        sampling_event = item.sampling_event_device.sampling_event
        collection = sampling_event.collection

        return collection.has_permission(
            user, 'add_collection_annotation_vote')



# -*- coding: utf-8 -*-
from irekua_models.models import ModelPrediction

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.models import model_version
from irekua_rest_api.serializers.items import items
from irekua_rest_api.serializers.object_types import events
from irekua_rest_api.serializers.terms import terms
from irekua_rest_api.serializers.users import users


class SelectSerializer(IrekuaModelSerializer):
    model_version = model_version.SelectSerializer(read_only=True)
    item = items.SelectSerializer(read_only=True)

    class Meta:
        model = ModelPrediction
        fields = ('url', 'id', 'item', 'model_version')


class ListSerializer(IrekuaModelSerializer):
    model_version = model_version.SelectSerializer(read_only=True)
    item = items.SelectSerializer(read_only=True)
    event_type = events.SelectSerializer(read_only=True)

    class Meta:
        model = ModelPrediction
        fields = (
            'url',
            'id',
            'item',
            'model_version',
            'event_type',
            'certainty'
        )


class DetailSerializer(IrekuaModelSerializer):
    model_version = model_version.ListSerializer(read_only=True)
    item = items.ListSerializer(read_only=True)
    event_type = events.ListSerializer(read_only=True)
    labels = terms.ListSerializer(read_only=True, many=True)
    created_by = users.SelectSerializer(read_only=True)
    modified_by = users.SelectSerializer(read_only=True)

    class Meta:
        model = ModelPrediction
        fields = (
            'url',
            'id',
            'item',
            'model_version',
            'event_type',
            'certainty',
            'labels',
            'annotation',
            'created_by',
            'modified_by',
            'created_on',
            'modified_on'
        )

        extra_kwargs = {
            'labels': {'style': {'base_template': 'input.html'}},
        }


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = ModelPrediction
        fields = (
            'item',
            'model_version',
            'event_type',
            'certainty',
            'labels',
            'annotation',
        )

        extra_kwargs = {
            'item': {'style': {'base_template': 'input.html'}},
            'labels': {'style': {'base_template': 'input.html'}},
        }

    def create(self, validated_data):
        user = self.context['request'].user
        validated_data['created_by'] = user
        validated_data['modified_by'] = user
        return super().create(validated_data)


class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = ModelPrediction
        fields = (
            'certainty',
            'labels',
            'annotation',
        )

        extra_kwargs = {
            'labels': {'style': {'base_template': 'input.html'}},
        }

    def update(self, validated_data):
        user = self.context['request'].user
        validated_data['modified_by'] = user
        return super().create(validated_data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import CollectionType
from irekua_database.models import EventType

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from irekua_rest_api.serializers.object_types import events
from . import types


MODEL = CollectionType.event_types.through  # pylint: disable=E1101


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = MODEL
        fields = (
            'url',
            'id',
        )


class ListSerializer(IrekuaModelSerializer):
    event_type = serializers.PrimaryKeyRelatedField(
        many=False,
        read_only=True,
        source='eventtype')

    class Meta:
        model = MODEL
        fields = (
            'url',
            'id',
            'event_type',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    event_type = events.SelectSerializer(
        many=False,
        read_only=True,
        source='eventtype')
    collection_type = types.SelectSerializer(
        many=False,
        read_only=True,
        source='collectiontype')

    class Meta:
        model = MODEL
        fields = (
            'url',
            'id',
            'collection_type',
            'event_type',
        )


class CreateSerializer(IrekuaModelSerializer):
    event_type = serializers.PrimaryKeyRelatedField(
        many=False,
        read_only=False,
        queryset=EventType.objects.all(),  # pylint: disable=E1101
        source='eventtype')

    class Meta:
        model = MODEL
        fields = (
            'event_type',
        )

    def create(self, validated_data):
        collection_type = self.context['collection_type']
        validated_data['collectiontype'] = collection_type
        return super().create(validated_data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.shortcuts import get_object_or_404
from django.db.models import Q
from django.contrib.auth.models import Permission
from django.shortcuts import redirect
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework import status
from rest_framework import mixins
from rest_framework.viewsets import GenericViewSet

from irekua_database import models
from irekua_rest_api import serializers
from irekua_rest_api import filters
from irekua_rest_api import utils

from irekua_rest_api.permissions import IsAuthenticated
from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import IsCurator
from irekua_rest_api.permissions import IsModel
from irekua_rest_api.permissions import IsSpecialUser
from irekua_rest_api.permissions import items as permissions


class ItemViewSet(mixins.UpdateModelMixin,
                  mixins.RetrieveModelMixin,
                  mixins.DestroyModelMixin,
                  mixins.ListModelMixin,
                  utils.CustomViewSetMixin,
                  GenericViewSet):
    queryset = models.Item.objects.all()  # pylint: disable=E1101
    filterset_class = filters.items.Filter
    search_fields = filters.items.search_fields

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.items.items)
        .extend(
            annotations=serializers.annotations.annotations.ListSerializer,
            add_annotation=serializers.annotations.annotations.CreateSerializer,
            types=serializers.object_types.items.ListSerializer,
            add_type=serializers.object_types.items.CreateSerializer,
            download=serializers.items.items.DownloadSerializer,
            upload=serializers.items.items.DownloadSerializer,
            tags=serializers.items.tags.ListSerializer,
            add_tag=serializers.items.tags.CreateSerializer,
            tag_item=serializers.items.tags.SelectSerializer,
            untag_item=serializers.items.tags.SelectSerializer,
            event_types=serializers.object_types.events.ListSerializer,
            add_event_type=serializers.object_types.events.CreateSerializer,
            secondary_items=serializers.items.secondary_items.ListSerializer,
            add_secondary_item=serializers.items.secondary_items.CreateSerializer,
        ))
    permission_mapping = utils.PermissionMapping({
        utils.Actions.UPDATE: [
            IsAuthenticated,
            (
                permissions.IsCreator |
                permissions.HasUpdatePermission |
                IsAdmin
            )
        ],
        utils.Actions.RETRIEVE: [
            IsAuthenticated,
            (
                permissions.IsCreator |
                permissions.HasViewPermission |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                permissions.ItemIsOpenToView |
                IsSpecialUser
            )
        ],
        utils.Actions.DESTROY: [
            IsAuthenticated,
            (
                permissions.IsCreator |
                IsAdmin
            )
        ],
        'annotations': [
            IsAuthenticated,
            (
                permissions.IsCreator |
                permissions.HasViewAnnotationsPermission |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                permissions.ItemIsOpenToViewAnnotations |
                IsSpecialUser
            )
        ],
        'add_annotation': [
            permissions.CanAnnotate,
        ],
        'tag_item': [
            IsAuthenticated,
            (
                permissions.IsCreator |
                IsCurator |
                IsAdmin
            )
        ],
        'remove_tag': [
            IsAuthenticated,
            (
                permissions.IsCreator |
                IsCurator |
                IsAdmin
            )
        ],
        'upload': [
            IsAuthenticated,
            (
                permissions.IsCreator |
                IsAdmin
            )
        ],
        'download': [
            IsAuthenticated,
            (
                permissions.IsCreator |
                permissions.HasDownloadPermission |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                permissions.ItemIsOpenToDownload |
                IsSpecialUser
            )
        ],
        'add_type': [IsAuthenticated, IsAdmin],
        'add_event_type': [IsAuthenticated, IsAdmin],
        'secondary_items': [
            IsAuthenticated,
            (
                permissions.IsCreator |
                permissions.HasViewPermission |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                permissions.ItemIsOpenToView |
                IsSpecialUser
            )
        ],
        'add_secondary_item': [
            IsAuthenticated,
            (
                permissions.IsCreator | # TODO: Add correct permissions
                IsSpecialUser
            )
        ]
    }, default=IsAuthenticated)

    def get_object(self):
        item_id = self.kwargs['pk']
        item = get_object_or_404(models.Item, pk=item_id)

        self.check_object_permissions(self.request, item)
        return item

    def get_serializer_context(self):
        context = super().get_serializer_context()

        try:
            item = self.get_object()
        except (KeyError, AssertionError, AttributeError):
            item = None

        context['item'] = item
        return context

    def get_queryset(self):
        if self.action == 'list':
            return self.get_list_queryset()

        if self.action == 'tags':
            return models.Tag.objects.all()  # pylint: disable=E1101

        if self.action == 'annotations':
            item_id = self.kwargs['pk']
            return models.Annotation.objects.filter(item=item_id)  # pylint: disable=E1101

        if self.action == 'secondary_items':
            item_id = self.kwargs['pk']
            return models.SecondaryItem.objects.filter(item=item_id)  # pylint: disable=E1101

        if self.action == 'types':
            return models.ItemType.objects.all()  # pylint: disable=E1101

        if self.action == 'event_types':
            return models.EventType.objects.all()  # pylint: disable=E1101

        return super().get_queryset()

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.tags.Filter,
        search_fields=filters.tags.search_fields)
    def tags(self, request):
        return self.list_related_object_view()

    @tags.mapping.post
    def add_tag(self, request):
        return self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.annotations.Filter,
        search_fields=filters.annotations.search_fields)
    def annotations(self, request, pk=None):
        return self.list_related_object_view()

    @annotations.mapping.post
    def add_annotation(self, request, pk=None):
        try:
            return self.create_related_object_view()
        except Exception as e:
            print(e)

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.secondary_items.Filter,
        search_fields=filters.secondary_items.search_fields)
    def secondary_items(self, request, pk=None):
        return self.list_related_object_view()

    @secondary_items.mapping.post
    def add_secondary_item(self, request, pk=None):
        return self.create_related_object_view()

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.item_types.Filter,
        search_fields=filters.item_types.search_fields)
    def types(self, request):
        return self.list_related_object_view()

    @types.mapping.post
    def add_type(self, request):
        return self.create_related_object_view()

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.event_types.Filter,
        search_fields=filters.event_types.search_fields)
    def event_types(self, request):
        return self.list_related_object_view()

    @event_types.mapping.post
    def add_event_type(self, request):
        return self.create_related_object_view()

    @action(detail=True, methods=['POST'])
    def tag_item(self, request, pk=None):
        return self.add_related_object_view(models.Tag, 'tag')

    @action(detail=True, methods=['POST'])
    def untag_item(self, request, pk=None):
        return self.remove_related_object_view('tag')

    @action(detail=True, methods=['POST'])
    def upload(self, request, pk=None):
        item = self.get_object()

        if item.item_file.name != '':
            return Response(
                'File previously uploaded',
                status=status.HTTP_403_FORBIDDEN)

        serializer = self.get_serializer(item, data=request.data, partial=True)

        if not serializer.is_valid():
            return Response('Invalid file', status=status.HTTP_400_BAD_REQUEST)

        serializer.save()
        return Response('File uploaded', status=status.HTTP_200_OK)

    @action(detail=True, methods=['GET'])
    def download(self, request, pk=None):
        item = self.get_object()

        if item.item_file.name == '':
            return Response(
                'File not uploaded to server',
                status=status.HTTP_404_NOT_FOUND)

        serializer_class = self.get_serializer_class()
        context = self.get_serializer_context()
        serializer = serializer_class(item, context=context)

        url = serializer.data['item_file']
        return redirect(url)

    @action(
        detail=True,
        methods=['GET'])
    def location(self, request, pk=None):
        item = self.get_object()
        serializer = serializers.sites.ItemLocationSerializer(
            [item],
            many=True)
        return Response(serializer.data)

    def get_list_queryset(self):
        try:
            user = self.request.user
        except AttributeError:
            return models.Item.objects.none()  # pylint: disable=E1101

        is_special_user = (
            user.is_superuser |
            user.is_curator |
            user.is_model |
            user.is_developer
        )
        if is_special_user:
            return self.get_full_queryset()

        return self.get_normal_queryset(user)

    def get_normal_queryset(self, user):
        is_open = (
            Q(licence__is_active=False) |
            Q(licence__licence_type__can_view=True)
        )
        is_owner = Q(created_by=user.pk)

        perm = Permission.objects.get(codename='view_collection_items')
        collections_with_permission = (
            models.CollectionUser.objects  # pylint: disable=E1101
            .filter(
                user=user.pk,
                role__in=perm.role_set.all()
            ).values('collection')
        )
        # TODO
        # Check that this query is working

        is_in_allowed_collection = Q(
            sampling_event_device__sampling_event__collection__in=collections_with_permission)

        filter_query = (
            is_open |
            is_owner |
            is_in_allowed_collection
        )

        queryset = models.Item.objects.filter(filter_query)  # pylint: disable=E1101
        return queryset

    def get_full_queryset(self):
        return models.Item.objects.all()  # pylint: disable=E1101



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import mixins
from rest_framework.viewsets import GenericViewSet

from irekua_database import models
from irekua_rest_api import utils
from irekua_rest_api import serializers

from irekua_rest_api.permissions import IsAuthenticated


class SecondaryItemViewSet(mixins.UpdateModelMixin,
                           mixins.RetrieveModelMixin,
                           mixins.DestroyModelMixin,
                           utils.CustomViewSetMixin,
                           GenericViewSet):
    queryset = models.SecondaryItem.objects.all()  # pylint: disable=E1101

    permission_mapping = utils.PermissionMapping(default=IsAuthenticated) # TODO: Fix permissions
    serializer_mapping = utils.SerializerMapping.from_module(
        serializers.items.secondary_items)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import Synonym

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from . import terms


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = Synonym
        fields = (
            'url',
            'id',
        )


class ListSerializer(IrekuaModelSerializer):
    source_type = serializers.CharField(
        read_only=True,
        source='source.term_type.name')
    source_value = serializers.CharField(
        read_only=True,
        source='source.value')
    target_type = serializers.CharField(
        read_only=True,
        source='target.term_type.name')
    target_value = serializers.CharField(
        read_only=True,
        source='target.value')

    class Meta:
        model = Synonym
        fields = (
            'url',
            'id',
            'source_type',
            'source_value',
            'target_type',
            'target_value',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    source = terms.SelectSerializer(many=False, read_only=True)
    target = terms.SelectSerializer(many=False, read_only=True)

    class Meta:
        model = Synonym
        fields = (
            'url',
            'metadata',
            'source',
            'target',
            'created_on',
            'modified_on',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = Synonym
        fields = (
            'source',
            'target',
            'metadata',
        )



class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = Synonym
        fields = (
            'metadata',
        )



from irekua_database.models import EntailmentType
from .utils import BaseFilter


search_fields = (
    'source_type__name',
    'target_type__name',
)


class Filter(BaseFilter):
    class Meta:
        model = EntailmentType
        fields = (
            'source_type__name',
            'target_type__name',
        )



from uuid import uuid4

from rest_framework.test import APITestCase
from irekua_database.models import TermType
from irekua_rest_api.serializers import terms
from .utils import (
    BaseTestCase,
    Users,
    Actions,
    create_permission_mapping_from_lists
)


class TermTestCase(BaseTestCase, APITestCase):
    serializer = terms.CreateSerializer
    permissions = create_permission_mapping_from_lists({
        Actions.LIST: Users.ALL_AUTHENTICATED_USERS,
        Actions.CREATE: [
            Users.ADMIN,
            Users.DEVELOPER],
        Actions.RETRIEVE: Users.ALL_AUTHENTICATED_USERS,
        Actions.UPDATE: [
            Users.ADMIN,
            Users.DEVELOPER],
        Actions.PARTIAL_UPDATE: [
            Users.ADMIN,
            Users.DEVELOPER],
        Actions.DESTROY: [
            Users.ADMIN,
            Users.DEVELOPER],
    })

    def setUp(self):
        super().setUp()
        TermType.objects.create(
            name='sample term type',
            description='random term type',
            is_categorical=True)

    @staticmethod
    def generate_random_json_data():
        data = {
            'term_type': 'sample term type',
            'value': str(uuid4()),
            'metadata': {},
        }
        return data



from rest_framework.permissions import BasePermission


class IsSigner(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        signer = obj.signed_by
        return user == signer



from rest_framework.permissions import BasePermission

from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import ReadOnly


class PermissionMapping(object):
    DEFAULT_PERMISSION = IsAdmin | ReadOnly

    def __init__(self, mapping=None, default=None):
        if mapping is None:
            mapping = {}

        assert isinstance(mapping, dict)

        self.permission_mapping = mapping

        if default is None:
            default = PermissionMapping.DEFAULT_PERMISSION

        if not isinstance(default, (tuple, list)):
            default = [default]

        self.default_permission = default

    def get_permissions(self, action):
        try:
            permissions = self.permission_mapping[action]
            if not isinstance(permissions, (list, tuple)):
                return [permissions]
            return permissions
        except KeyError:
            return self.default_permission

    def extend(self, additional_actions=None, **kwargs):
        if additional_actions is None:
            additional_actions = {}

        extended_mapping = self.permission_mapping.copy()
        extended_mapping.update(additional_actions)

        for key in kwargs:
            extended_mapping[key] = kwargs[key]

        return PermissionMapping(extended_mapping)


class PermissionMappingMixin(object):
    @property
    def permission_mapping(self):
        print(self.__name__)
        raise NotImplementedError

    def get_permissions(self):
        if self.action is None:
            return []

        permission_classes = self.permission_mapping.get_permissions(self.action)
        return [permission() for permission in permission_classes]



from uuid import uuid4

from rest_framework.test import APITestCase

from irekua_database.models import User, DeviceType, Device, DeviceBrand
from irekua_database.utils import simple_JSON_schema
from irekua_rest_api.serializers import physical_devices
from .utils import (
    BaseTestCase,
    Users,
    Actions,
    create_permission_mapping_from_lists
)


class DummyRequest(object):
    def __init__(self, user):
        self.user = user


class PhysicalDevicesTestCase(BaseTestCase, APITestCase):
    serializer = physical_devices.CreateSerializer
    permissions = create_permission_mapping_from_lists({
        Actions.LIST: Users.ALL_AUTHENTICATED_USERS,
        Actions.CREATE: Users.ALL_AUTHENTICATED_USERS,
        Actions.RETRIEVE: [
            Users.ADMIN,
        ],
        Actions.UPDATE: [
            Users.ADMIN,
        ],
        Actions.PARTIAL_UPDATE: [
            Users.ADMIN,
        ],
        Actions.DESTROY: [
            Users.ADMIN,
        ],
    })

    def setUp(self):
        super(PhysicalDevicesTestCase, self).setUp()
        self.random_user = User.objects.create(
            username=str(uuid4()),
            password=str(uuid4()))

        device_type = DeviceType.objects.create(
            name=str(uuid4()),
            description='Random device type')

        device_brand = DeviceBrand.objects.create(
            name=str(uuid4())
        )

        self.device = Device.objects.create(
            device_type=device_type,
            brand=device_brand,
            model=str(uuid4()),
            metadata_schema=simple_JSON_schema(),
            configuration_schema=simple_JSON_schema())

    def get_serializer_context(self):
        request = DummyRequest(self.random_user)
        return {'request': request}

    def generate_random_json_data(self):
        data = {
            'device': self.device.pk,
            'serial_number': str(uuid4()),
            'metadata': {},
            'bundle': False,
        }
        return data



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from irekua_database.models import AnnotationTool


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = AnnotationTool
        fields = (
            'url',
            'name',
        )


class ListSerializer(IrekuaModelSerializer):
    class Meta:
        model = AnnotationTool
        fields = (
            'url',
            'name',
            'version',
            'logo',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    class Meta:
        model = AnnotationTool
        fields = (
            'url',
            'id',
            'name',
            'version',
            'logo',
            'website',
            'created_on',
            'modified_on'
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = AnnotationTool
        fields = (
            'name',
            'version',
            'logo',
            'website',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import CollectionSite

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from irekua_rest_api.serializers import sites
from . import data_collections


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = CollectionSite
        fields = (
            'url',
            'id',
        )


class ListSerializer(IrekuaModelSerializer):
    locality = serializers.SlugRelatedField(
        many=False,
        read_only=True,
        source='site',
        slug_field='locality')

    class Meta:
        model = CollectionSite
        fields = (
            'url',
            'id',
            'site',
            'site_type',
            'locality',
            'internal_id',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    site = sites.SelectSerializer(
        many=False,
        read_only=True)
    collection = data_collections.SelectSerializer(
        many=False,
        read_only=True)

    class Meta:
        model = CollectionSite
        fields = (
            'url',
            'id',
            'collection',
            'site',
            'internal_id',
            'created_on',
            'modified_on',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = CollectionSite
        fields = (
            'site',
            'internal_id',
        )

    def create(self, validated_data):
        collection = self.context['collection']
        validated_data['collection'] = collection
        return super().create(validated_data)



# -*- coding: utf-8 -*-
from irekua_models.models import ModelVersion

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.models import model


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = ModelVersion
        fields = ('url', 'id', 'model', 'version')


class ListSerializer(IrekuaModelSerializer):
    model = model.SelectSerializer(read_only=True)

    class Meta:
        model = ModelVersion
        fields = (
            'url',
            'id',
            'model',
            'version',
        )


class DetailSerializer(IrekuaModelSerializer):
    model = model.ListSerializer(read_only=True)

    class Meta:
        model = ModelVersion
        fields = (
            'url',
            'id',
            'model',
            'version',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = ModelVersion
        fields = ('model', 'version')

    def create(self, validated_data):
        user = self.context['request'].user
        validated_data['created_by'] = user
        validated_data['modified_by'] = user
        return super().create(validated_data)


class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = ModelVersion
        fields = ('version',)

    def update(self, validated_data):
        user = self.context['request'].user
        validated_data['modified_by'] = user
        return super().create(validated_data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import mixins
from rest_framework.viewsets import GenericViewSet

from irekua_database import models
from irekua_rest_api import serializers
from irekua_rest_api import utils

from irekua_rest_api.permissions import IsAuthenticated
from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import licences as permissions


class LicenceViewSet(mixins.UpdateModelMixin,
                     mixins.RetrieveModelMixin,
                     mixins.DestroyModelMixin,
                     utils.CustomViewSetMixin,
                     GenericViewSet):
    queryset = models.Licence.objects.all()  # pylint: disable=E1101

    serializer_mapping = utils.SerializerMapping.from_module(serializers.licences)

    permission_mapping = utils.PermissionMapping({
        utils.Actions.UPDATE: [
            IsAuthenticated,
            (
                permissions.IsSigner |
                IsAdmin
            )
        ],
        utils.Actions.RETRIEVE: IsAuthenticated,
        utils.Actions.DESTROY: [
            IsAuthenticated,
            (
                permissions.IsSigner |
                IsAdmin
            )
        ],
    })



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework.viewsets import ModelViewSet
from rest_framework.decorators import action

from irekua_database import models
from irekua_rest_api import filters
from irekua_rest_api import utils
from irekua_rest_api import serializers

from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import IsAuthenticated


class DeviceViewSet(utils.CustomViewSetMixin, ModelViewSet):
    queryset = models.Device.objects.all()  # pylint: disable=E1101
    filterset_class = filters.devices.Filter
    search_fields = filters.devices.search_fields

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.devices.devices)
        .extend(
            types=serializers.object_types.devices.ListSerializer,
            add_type=serializers.object_types.devices.CreateSerializer,
            brands=serializers.devices.brands.ListSerializer,
            add_brand=serializers.devices.brands.CreateSerializer,
            physical_devices=serializers.devices.physical_devices.ListSerializer,
            add_physical_device=serializers.devices.physical_devices.CreateSerializer,
        ))

    permission_mapping = utils.PermissionMapping({
        utils.Actions.UPDATE: IsAdmin,
        utils.Actions.DESTROY: IsAdmin,
        'add_type': IsAdmin,
    }, default=IsAuthenticated)

    def get_queryset(self):
        if self.action == 'types':
            return models.DeviceType.objects.all()  # pylint: disable=E1101

        if self.action == 'brands':
            return models.DeviceBrand.objects.all()  # pylint: disable=E1101

        if self.action == 'physical_devices':
            # TODO: set adequate queryset for user
            return models.PhysicalDevice.objects.all() # pylint: disable=E1101

        return super().get_queryset()

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.device_types.Filter,
        search_fields=filters.device_types.search_fields)
    def types(self, request):
        return self.list_related_object_view()

    @types.mapping.post
    def add_type(self, request):
        return self.create_related_object_view()

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.device_brands.Filter,
        search_fields=filters.device_brands.search_fields)
    def brands(self, request):
        return self.list_related_object_view()

    @brands.mapping.post
    def add_brand(self, request):
        return self.create_related_object_view()

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.physical_devices.Filter,
        search_fields=filters.physical_devices.search_fields)
    def physical_devices(self, request):
        return self.list_related_object_view()

    @physical_devices.mapping.post
    def add_physical_device(self, request):
        return self.create_related_object_view()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import mixins
from rest_framework.decorators import action
from rest_framework.viewsets import GenericViewSet

from irekua_database.models import Term
from irekua_models import models
from irekua_rest_api import utils
from irekua_rest_api import filters
from irekua_rest_api import serializers

from irekua_rest_api.permissions import IsAuthenticated


class ModelViewSet(
        mixins.ListModelMixin,
        mixins.CreateModelMixin,
        mixins.UpdateModelMixin,
        mixins.RetrieveModelMixin,
        mixins.DestroyModelMixin,
        utils.CustomViewSetMixin,
        GenericViewSet):
    queryset = models.Model.objects.all()  # pylint: disable=no-member
    filterset_class = filters.models.Filter
    search_fields = filters.models.search_fields

    permission_mapping = utils.PermissionMapping(default=IsAuthenticated)
    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.models.model)
        .extend(
            terms=serializers.terms.terms.ListSerializer,
            add_term=serializers.terms.terms.SelectSerializer)
    )

    def get_queryset(self):
        if self.action == 'terms':
            return Term.objects.filter(model=self.kwargs['pk'])

        return super().get_queryset()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.shortcuts import get_object_or_404
from rest_framework import mixins
from rest_framework.viewsets import GenericViewSet
from rest_framework.decorators import action
from rest_framework.response import Response

from irekua_database import models
from irekua_rest_api import serializers
from irekua_rest_api import filters
from irekua_rest_api import utils

from irekua_rest_api.permissions import IsAuthenticated
from irekua_rest_api.permissions import IsAdmin


class SamplingEventDeviceViewSet(mixins.UpdateModelMixin,
                                 mixins.RetrieveModelMixin,
                                 mixins.DestroyModelMixin,
                                 utils.CustomViewSetMixin,
                                 GenericViewSet):
    queryset = models.SamplingEventDevice.objects.all()  # pylint: disable=E1101
    filterset_class = filters.sampling_event_devices.Filter
    search_fields = filters.sampling_event_devices.search_fields

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.sampling_events.devices)
        .extend(
            items=serializers.items.items.ListSerializer,
            add_item=serializers.items.items.CreateSerializer,
        ))

    permission_mapping = utils.PermissionMapping({
        utils.Actions.RETRIEVE: IsAuthenticated # TODO: Fix permissions
    }, default=[IsAuthenticated])

    def get_object(self):
        device_pk = self.kwargs['pk']
        device = get_object_or_404(models.SamplingEventDevice, pk=device_pk)

        self.check_object_permissions(self.request, device)
        return device

    def get_serializer_context(self):
        context = super().get_serializer_context()

        try:
            sampling_event_device = self.get_object()
        except (KeyError, AttributeError, AssertionError):
            sampling_event_device = None

        context['sampling_event_device'] = sampling_event_device
        return context

    def get_queryset(self):
        if self.action == 'items':
            object_id = self.kwargs['pk']
            return models.Item.objects.filter(sampling_event_device=object_id)  # pylint: disable=E1101

        return super().get_queryset()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.items.Filter,
        search_fields=filters.items.search_fields)
    def items(self, request, pk=None):
        return self.list_related_object_view()

    @items.mapping.post
    def add_item(self, request, pk=None):
        self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'])
    def location(self, request, pk=None):
        sampling_event_device = self.get_object()
        serializer = serializers.sites.SamplingEventDeviceLocationSerializer(
            [sampling_event_device],
            many=True)

        return Response(serializer.data)



from uuid import uuid4

from rest_framework.test import APITestCase
from irekua_database.utils import simple_JSON_schema
from irekua_rest_api.serializers import sampling_event_types
from .utils import (
    BaseTestCase,
    Users,
    Actions,
    create_permission_mapping_from_lists
)


class SamplingEventTypeTestCase(BaseTestCase, APITestCase):
    serializer = sampling_event_types.CreateSerializer
    permissions = create_permission_mapping_from_lists({
        Actions.LIST: Users.ALL_AUTHENTICATED_USERS,
        Actions.CREATE: [
            Users.ADMIN,
            ],
        Actions.RETRIEVE: Users.ALL_AUTHENTICATED_USERS,
        Actions.UPDATE: [
            Users.ADMIN,
            ],
        Actions.PARTIAL_UPDATE: [
            Users.ADMIN,
            ],
        Actions.DESTROY: [
            Users.ADMIN,
            ],
        })

    @staticmethod
    def generate_random_json_data():
        data = {
            'name': str(uuid4()),
            'description': 'Random sampling event type',
            'metadata_schema': simple_JSON_schema(),
            'restrict_device_types': False,
            'restrict_site_types': False
        }
        return data



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.shortcuts import get_object_or_404
from rest_framework import mixins
from rest_framework.viewsets import GenericViewSet
from rest_framework.decorators import action
from rest_framework.response import Response

from irekua_database import models
from irekua_rest_api import serializers
from irekua_rest_api import filters
from irekua_rest_api import utils

from irekua_rest_api.permissions import IsAuthenticated
from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import IsSpecialUser
import irekua_rest_api.permissions.sampling_events as permissions


class SamplingEventViewSet(mixins.UpdateModelMixin,
                           mixins.RetrieveModelMixin,
                           mixins.DestroyModelMixin,
                           mixins.ListModelMixin,
                           utils.CustomViewSetMixin,
                           GenericViewSet):
    queryset = models.SamplingEvent.objects.all()  # pylint: disable=E1101
    search_fields = filters.sampling_events.search_fields
    filterset_class = filters.sampling_events.Filter

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.sampling_events.sampling_events)
        .extend(
            items=serializers.items.items.ListSerializer,
            devices=serializers.sampling_events.devices.ListSerializer,
            add_device=serializers.sampling_events.devices.CreateSerializer,
        ))

    permission_mapping = utils.PermissionMapping({
        utils.Actions.UPDATE: [
            IsAuthenticated,
            (
                permissions.IsCreator |
                permissions.HasChangePermissions |
                IsAdmin
            )
        ],
        utils.Actions.RETRIEVE: [
            IsAuthenticated,
            (
                permissions.IsCreator |
                permissions.HasViewPermissions |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                IsSpecialUser
            )
        ],
        utils.Actions.DESTROY: [
            IsAuthenticated,
            (
                permissions.IsCreator |
                IsAdmin
            )
        ],
        utils.Actions.LIST: [
            IsAuthenticated,
        ],
        'items': [
            IsAuthenticated,
            (
                permissions.IsCreator |
                permissions.HasViewItemsPermissions |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                IsAdmin
            )
        ],
    })

    def get_object(self):
        sampling_event_pk = self.kwargs['pk']
        sampling_event = get_object_or_404(
            models.SamplingEvent,
            pk=sampling_event_pk)

        self.check_object_permissions(self.request, sampling_event)
        return sampling_event

    def get_serializer_context(self):
        context = super().get_serializer_context()

        try:
            sampling_event = self.get_object()
        except (KeyError, AssertionError, AttributeError):
            sampling_event = None

        context['sampling_event'] = sampling_event
        return context

    def get_queryset(self):
        if self.action == 'devices':
            object_id = self.kwargs['pk']
            return models.SamplingEventDevice.objects.filter(sampling_event=object_id)  # pylint: disable=E1101

        if self.action == 'items' or self.action == 'item_locations':
            object_id = self.kwargs['pk']
            return models.Item.objects.filter(  # pylint: disable=E1101
                sampling_event_device__sampling_event=object_id)

        return super().get_queryset()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.sampling_event_devices.Filter,
        search_fields=filters.sampling_event_devices.search_fields)
    def devices(self, request, pk=None):
        return self.list_related_object_view()

    @devices.mapping.post
    def add_device(self, request, pk=None):
        return self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.items.Filter,
        search_fields=filters.items.search_fields)
    def items(self, request, pk=None):
        return self.list_related_object_view()

    @action(
        detail=True,
        methods=['GET'])
    def location(self, request, pk=None):
        sampling_event = self.get_object()
        serializer = serializers.sites.SamplingEventLocationSerializer(
            [sampling_event],
            many=True)

        return Response(serializer.data)

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.items.Filter,
        search_fields=filters.items.search_fields)
    def item_locations(self, request, pk=None):
        queryset = self.filter_queryset(self.get_queryset())

        serializer = serializers.sites.ItemLocationSerializer(
            queryset,
            many=True,
            read_only=True)
        return Response(serializer.data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import EventType

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from . import terms


class SelectSerializer(IrekuaModelSerializer):
    name = serializers.PrimaryKeyRelatedField(
        many=False,
        read_only=False,
        queryset=EventType.objects.all())

    class Meta:
        model = EventType
        fields = (
            'url',
            'name',
            'id',
        )


class DescriptionSerializer(IrekuaModelSerializer):
    class Meta:
        model = EventType
        fields = (
            'name',
            'description',
        )


class ListSerializer(IrekuaModelSerializer):
    class Meta:
        model = EventType
        fields = (
            'url',
            'id',
            'name',
            'description',
            'icon',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    term_types = terms.SelectSerializer(
        many=True,
        read_only=True)

    class Meta:
        model = EventType
        fields = (
            'url',
            'id',
            'name',
            'description',
            'icon',
            'term_types',
            'created_on',
            'modified_on'
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = EventType
        fields = (
            'name',
            'description',
            'icon',
        )


class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = EventType
        fields = (
            'description',
            'icon',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from irekua_database.models import ItemType

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from . import events
from . import mime_types


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = ItemType
        fields = (
            'url',
            'name',
            'id',
        )


class ListSerializer(IrekuaModelSerializer):
    class Meta:
        model = ItemType
        fields = (
            'url',
            'name',
            'id',
            'description',
            'icon',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    event_types = events.DetailSerializer(
        many=True,
        read_only=True)
    mime_types = mime_types.SelectSerializer(
        many=True,
        read_only=True)

    class Meta:
        model = ItemType
        fields = (
            'url',
            'name',
            'id',
            'description',
            'mime_types',
            'icon',
            'event_types',
            'created_on',
            'modified_on',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = ItemType
        fields = (
            'name',
            'description',
            'icon',
        )


class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = ItemType
        fields = (
            'description',
            'icon',
        )



from rest_framework.permissions import BasePermission


class IsCreator(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        creator = obj.created_by
        return user == creator



from django import forms
from django.utils.translation import gettext as _
import django_filters


class BaseFilter(django_filters.FilterSet):
    created_on__lt = django_filters.DateTimeFilter(
        field_name='created_on',
        lookup_expr='lt',
        label=_('Created before than'),
        widget=forms.DateTimeInput)
    created_on__gt = django_filters.DateTimeFilter(
        field_name='created_on',
        lookup_expr='gt',
        label=_('Created after than'),
        widget=forms.DateTimeInput)
    modified_on__lt = django_filters.DateTimeFilter(
        field_name='modified_on',
        lookup_expr='lt',
        label=_('Modified before than'),
        widget=forms.DateTimeInput)
    modified_on__gt = django_filters.DateTimeFilter(
        field_name='modified_on',
        lookup_expr='gt',
        label=_('Modified after than'),
        widget=forms.DateTimeInput)



from irekua_database.models import AnnotationVote
from .utils import BaseFilter


search_fields = (
    'created_by__username',
)


class Filter(BaseFilter):
    class Meta:
        model = AnnotationVote
        fields = (
            'created_by__username',
            'created_by__first_name',
            'created_by__last_name',
        )



from django import forms
import django_filters

from irekua_database.models import Licence

search_fields = (
    'licence_type',
)


class Filter(django_filters.FilterSet):
    created_on__lt = django_filters.DateTimeFilter(
        field_name='created_on',
        lookup_expr='lt',
        widget=forms.DateTimeInput)
    created_on__gt = django_filters.DateTimeFilter(
        field_name='created_on',
        lookup_expr='gt',
        widget=forms.DateTimeInput)

    class Meta:
        model = Licence
        fields = (
            'licence_type',
            'is_active',
            'created_by__username',
            'created_by__first_name',
            'created_by__last_name',
            'licence_type__years_valid_for',
            'licence_type__can_view',
            'licence_type__can_download',
            'licence_type__can_view_annotations',
            'licence_type__can_annotate',
            'licence_type__can_vote_annotations',
        )



from .actions import Actions


class SerializerMapping(object):
    DEFAULT_ACTION = Actions.CREATE

    def __init__(self, mapping):
        assert isinstance(mapping, dict)
        assert self.DEFAULT_ACTION in mapping
        self.serializer_mapping = mapping

    def get_serializer_class(self, action):
        try:
            return self.serializer_mapping[action]
        except KeyError:
            return self.serializer_mapping[self.DEFAULT_ACTION]

    @classmethod
    def from_module(cls, module):
        serializer_mapping = {}

        if hasattr(module, 'ListSerializer'):
            serializer_mapping[Actions.LIST] = module.ListSerializer

        if hasattr(module, 'CreateSerializer'):
            serializer_mapping[Actions.CREATE] = module.CreateSerializer

        if hasattr(module, 'DetailSerializer'):
            serializer_mapping[Actions.RETRIEVE] = module.DetailSerializer

        if hasattr(module, 'UpdateSerializer'):
            serializer_mapping[Actions.UPDATE] = module.UpdateSerializer
            serializer_mapping[Actions.PARTIAL_UPDATE] = module.UpdateSerializer

        if hasattr(module, 'MetadataSerializer'):
            serializer_mapping[Actions.METADATA] = module.MetadataSerializer

        return cls(serializer_mapping)

    def extend(self, additional_actions=None, **kwargs):
        if additional_actions is None:
            additional_actions = {}

        extended_mapping = self.serializer_mapping.copy()
        extended_mapping.update(additional_actions)

        for key in kwargs:
            extended_mapping[key] = kwargs[key]

        return SerializerMapping(extended_mapping)


class SerializerMappingMixin(object):
    @property
    def serializer_mapping(self):
        raise NotImplementedError

    def get_serializer_class(self):
        return self.serializer_mapping.get_serializer_class(self.action)



from rest_framework.metadata import SimpleMetadata


class CustomMetadata(SimpleMetadata):
    def determine_actions(self, request, view):
        actions = {}
        for method, action in view.action_map.items():
            if method.lower() == "delete":
                continue

            view.action = action
            serializer = view.get_serializer()
            actions[method.upper()] = self.get_serializer_info(serializer)

        return actions



from django.utils.translation import gettext as _
import django_filters

from irekua_database.models import Site

from .utils import BaseFilter


search_fields = (
    'name',
    'locality',
)


class SiteBaseFilter(BaseFilter):
    latitude__gt = django_filters.NumberFilter(
        field_name='latitude',
        label=_('Latitude greater than'),
        lookup_expr='gt')
    latitude__lt = django_filters.NumberFilter(
        field_name='latitude',
        label=_('Latitude less than'),
        lookup_expr='lt')

    longitude__gt = django_filters.NumberFilter(
        field_name='longitude',
        label=_('Longitude greater than'),
        lookup_expr='gt')
    longitude__lt = django_filters.NumberFilter(
        field_name='longitude',
        label=_('Longitude less than'),
        lookup_expr='lt')

    altitude__gt = django_filters.NumberFilter(
        field_name='altitude',
        label=_('Altitude less than'),
        lookup_expr='gt')
    altitude__lt = django_filters.NumberFilter(
        field_name='altitude',
        label=_('Altitude less than'),
        lookup_expr='lt')

    class Meta:
        model = Site
        fields = (
            'name',
            'locality',
        )


class Filter(SiteBaseFilter):
    class Meta:
        model = Site
        fields = (
            'created_by',
        )

class UserFilter(SiteBaseFilter):
    class Meta:
        model = Site
        fields = (
            'name',
            'locality',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import CollectionDevice

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from irekua_rest_api.serializers.devices import physical_devices


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = CollectionDevice
        fields = (
            'url',
            'id',
        )


class ListSerializer(IrekuaModelSerializer):
    physical_device = serializers.StringRelatedField(
        many=False,
        read_only=True)

    class Meta:
        model = CollectionDevice
        fields = (
            'url',
            'id',
            'physical_device',
            'internal_id',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    physical_device = physical_devices.SelectSerializer(many=False, read_only=True)

    class Meta:
        model = CollectionDevice
        fields = (
            'url',
            'physical_device',
            'internal_id',
            'metadata',
            'created_on',
            'modified_on',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = CollectionDevice
        fields = (
            'physical_device',
            'metadata',
            'internal_id',
        )

    def create(self, validated_data):
        collection = self.context['collection']
        validated_data['collection'] = collection
        return super().create(validated_data)



from abc import ABCMeta
from uuid import uuid4
from six import add_metaclass

from rest_framework import status
from rest_framework import serializers
from rest_framework.permissions import SAFE_METHODS
from django.urls import reverse
from django.contrib.auth.models import User

from irekua_database.models import User
from irekua_database.tests.test_data_collections import create_simple_collection
from irekua_database.tests.test_collection_roles import create_simple_role


def create_or_get_user(
        is_superuser=None,
        is_developer=None,
        is_curator=None,
        is_model=None):

    user, _ = User.objects.get_or_create(
        username=uuid4(),
        defaults=dict(
            password=uuid4(),
            is_superuser=is_superuser,
            is_developer=is_developer,
            is_curator=is_curator,
            is_model=is_model)
    )

    return user


class Users:
    NON_AUTHENTICATED = 'non authenticated'
    USER = 'user'
    ADMIN = 'admin'
    MODEL = 'model'
    DEVELOPER = 'developer'
    CURATOR = 'curator'
    COLLECTION_USER = 'collection user'

    ALL_USERS = [
        NON_AUTHENTICATED,
        USER,
        ADMIN,
        MODEL,
        DEVELOPER,
        CURATOR,
        COLLECTION_USER,
    ]

    ALL_AUTHENTICATED_USERS = [
        USER,
        ADMIN,
        MODEL,
        DEVELOPER,
        CURATOR,
        COLLECTION_USER,
    ]


class Actions:
    LIST = 'list'
    CREATE = 'create'
    RETRIEVE = 'retrieve'
    UPDATE = 'update'
    PARTIAL_UPDATE = 'partial_update'
    DESTROY = 'destroy'

    ALL_ACTIONS = [
        LIST,
        CREATE,
        RETRIEVE,
        UPDATE,
        PARTIAL_UPDATE,
        DESTROY
    ]

    METHOD_MAPPING = {
        LIST: 'get',
        CREATE: 'post',
        RETRIEVE: 'get',
        UPDATE: 'put',
        PARTIAL_UPDATE: 'patch',
        DESTROY: 'delete'
    }

    STATUS_CODE_MAPPING = {
        LIST: status.HTTP_200_OK,
        CREATE: status.HTTP_201_CREATED,
        RETRIEVE: status.HTTP_200_OK,
        UPDATE: status.HTTP_200_OK,
        PARTIAL_UPDATE: status.HTTP_200_OK,
        DESTROY: status.HTTP_204_NO_CONTENT,
    }


def create_permission_mapping_from_lists(permission_lists):
    permission_mapping = {}

    for action in permission_lists:
        permission_mapping[action] = {}

        permission_list = permission_lists[action]
        for user_type in Users.ALL_USERS:
            permission_mapping[action][user_type] = (
                user_type in permission_list)

    return permission_mapping


@add_metaclass(ABCMeta)
class BaseTestCase(object):
    VIEW_NAME_MAPPING = {
        Actions.LIST: 'list',
        Actions.CREATE: 'list',
        Actions.RETRIEVE: 'detail',
        Actions.UPDATE: 'detail',
        Actions.PARTIAL_UPDATE: 'detail',
        Actions.DESTROY: 'detail'
    }

    @property
    def serializer(self):
        raise NotImplementedError

    @property
    def permissions(self):
        raise NotImplementedError

    @staticmethod
    def generate_random_json_data():
        raise NotImplementedError

    def setUp(self):
        self.admin_user = create_or_get_user(
            is_superuser=True,
            is_developer=False,
            is_curator=False,
            is_model=False)

        self.regular_user = create_or_get_user(
            is_superuser=False,
            is_developer=False,
            is_curator=False,
            is_model=False)

        self.model_user = create_or_get_user(
            is_superuser=False,
            is_developer=False,
            is_curator=False,
            is_model=True)

        self.developer_user = create_or_get_user(
            is_superuser=False,
            is_developer=True,
            is_curator=False,
            is_model=False)

        self.curator_user = create_or_get_user(
            is_superuser=False,
            is_developer=False,
            is_curator=True,
            is_model=False)

        self.collection = create_simple_collection()
        self.role = create_simple_role()
        self.collection.collection_type.add_role(self.role)

        self.collection_user = create_or_get_user(
            is_superuser=False,
            is_developer=False,
            is_curator=False,
            is_model=False)

        self.collection.add_user(
            self.collection_user,
            role=self.role,
            metadata={})

    def change_user(self, usertype):
        if usertype is Users.NON_AUTHENTICATED:
            self.client.force_authenticate()
            return

        mapping = {
            Users.USER: self.regular_user,
            Users.ADMIN: self.admin_user,
            Users.MODEL: self.model_user,
            Users.CURATOR: self.curator_user,
            Users.DEVELOPER: self.developer_user,
            Users.COLLECTION_USER: self.collection_user
        }

        self.client.force_authenticate(user=mapping[usertype])

    def check_response(self, action, response, has_permission, user_type):
        if not has_permission:
            self.assertEqual(
                response.status_code,
                status.HTTP_403_FORBIDDEN,
                msg=user_type)
        else:
            status_code = Actions.STATUS_CODE_MAPPING[action]
            self.assertEqual(response.status_code, status_code, msg=user_type)

    def check_permissions_for_action_and_users(self, action, permissions):
        method_name = self.get_http_method(action)
        method = getattr(self.client, method_name)
        url_name = self.get_url_name(action)

        for user_type in Users.ALL_USERS:

            if 'list' in url_name:
                url = reverse(url_name)
            else:
                sample_object = self.create_random_object()
                url = reverse(url_name, args=[sample_object.pk])

            self.change_user(user_type)
            if method_name.upper() in SAFE_METHODS:
                response = method(url)
            else:
                random_data = self.generate_random_json_data()
                response = method(url, random_data, format='json')

            self.check_response(
                action,
                response,
                permissions[user_type],
                user_type)

    @staticmethod
    def get_http_method(action):
        return Actions.METHOD_MAPPING[action]

    def get_serializer_context(self):
        return {}

    def create_random_object(self):
        context = self.get_serializer_context()
        data = self.generate_random_json_data()
        serializer_instance = self.serializer(
            data=data,
            context=context)
        if serializer_instance.is_valid():
            random_object = serializer_instance.save()
            return random_object
        else:
            raise serializers.ValidationError(serializer_instance.errors)

    def get_class_name(self):
        return self.serializer.Meta.model.__name__.lower()

    def get_url_name(self, action, view_name=None):
        class_name = self.get_class_name()
        base_url = 'rest-api:{class_name}-{view_name}'

        if view_name is None:
            view_name = self.VIEW_NAME_MAPPING[action]

        url_name = base_url.format(
            class_name=class_name,
            view_name=view_name)

        return url_name

    def test_list(self):
        action = Actions.LIST
        permissions = self.permissions[action]

        self.check_permissions_for_action_and_users(action, permissions)

    def test_create(self):
        action = Actions.CREATE
        permissions = self.permissions[action]

        self.check_permissions_for_action_and_users(action, permissions)

    def test_retrieve(self):
        action = Actions.RETRIEVE
        permissions = self.permissions[action]

        self.check_permissions_for_action_and_users(action, permissions)

    def test_update(self):
        action = Actions.UPDATE
        permissions = self.permissions[action]

        self.check_permissions_for_action_and_users(action, permissions)

    def test_partial_update(self):
        action = Actions.PARTIAL_UPDATE
        permissions = self.permissions[action]

        self.check_permissions_for_action_and_users(action, permissions)

    def test_destroy(self):
        action = Actions.DESTROY
        permissions = self.permissions[action]

        self.check_permissions_for_action_and_users(action, permissions)



IREKUA_REST_API_APPS = [
    'irekua_rest_api',
    'rest_framework',
    'django_filters',
]

REST_FRAMEWORK = {
    'DEFAULT_PAGINATION_CLASS': 'irekua_rest_api.pagination.StandardResultsSetPagination',
    'DEFAULT_VERSIONING_CLASS': 'rest_framework.versioning.NamespaceVersioning',
    'DEFAULT_METADATA_CLASS': 'irekua_rest_api.metadata.CustomMetadata',
    'DEFAULT_FILTER_BACKENDS': [
        'rest_framework.filters.SearchFilter',
        'django_filters.rest_framework.DjangoFilterBackend',
    ],
    'DEFAULT_PERMISSION_CLASSES': [
        'rest_framework.permissions.IsAuthenticated',
    ],
    'PAGE_SIZE': 10,
    'EXCEPTION_HANDLER': 'irekua_rest_api.exception_handler.custom_exception_handler',
    'DEFAULT_SCHEMA_CLASS': 'irekua_rest_api.utils.schemas.CustomSchema'
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import mixins
from rest_framework.viewsets import GenericViewSet

from irekua_models import models
from irekua_rest_api import utils
from irekua_rest_api import filters
from irekua_rest_api import serializers

from irekua_rest_api.permissions import IsAuthenticated


class ModelPredictionViewSet(
        mixins.ListModelMixin,
        mixins.CreateModelMixin,
        mixins.UpdateModelMixin,
        mixins.RetrieveModelMixin,
        mixins.DestroyModelMixin,
        utils.CustomViewSetMixin,
        GenericViewSet):
    # pylint: disable=no-member
    queryset = models.ModelPrediction.objects.all()
    filterset_class = filters.model_predictions.Filter
    search_fields = filters.model_predictions.search_fields

    permission_mapping = utils.PermissionMapping(default=IsAuthenticated)
    serializer_mapping = utils.SerializerMapping.from_module(
        serializers.models.model_prediction)



from irekua_database.models import User
from .utils import BaseFilter


search_fields = (
    'username',
    'first_name',
    'last_name',
)


class Filter(BaseFilter):
    class Meta:
        model = User
        fields = (
            'username',
            'first_name',
            'last_name',
            'institution__institution_name',
            'institution__institution_code',
            'institution__subdependency',
            'is_superuser',
            'is_curator',
            'is_model',
            'is_developer',
        )



from rest_framework.permissions import BasePermission


class IsCreator(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        return obj.created_by == user


class IsOpen(BasePermission):
    def has_object_permission(self, request, view, obj):
        annotation = obj.annotation
        item = annotation.item
        licence = item.licence

        if not licence.is_active:
            return True

        licence_type = licence.licence_type
        can_view = licence_type.can_view_annotations
        return can_view


class HasViewPermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        annotation = obj.annotation
        item = annotation.item
        sampling_event = item.sampling_event
        collection = sampling_event.collection

        try:
            collectionuser = collection.collectionuser_set.get(user=user)
        except collection.collectionuser_set.DoesNotExist:
            return False

        role = collectionuser.role
        has_permission = role.permissions.filter(
            codename='view_collection_annotation_vote').exists()
        return has_permission



from irekua_database.models import TermType
from .utils import BaseFilter


search_fields = (
    'name',
)


class Filter(BaseFilter):
    class Meta:
        model = TermType
        fields = ('name', )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from irekua_database.models import EntailmentType

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from irekua_rest_api.serializers.object_types import terms


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = EntailmentType


class ListSerializer(IrekuaModelSerializer):
    class Meta:
        model = EntailmentType
        fields = (
            'url',
            'id',
            'source_type',
            'target_type',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    source_type = terms.SelectSerializer(many=False, read_only=True)
    target_type = terms.SelectSerializer(many=False, read_only=True)

    class Meta:
        model = EntailmentType
        fields = (
            'url',
            'id',
            'source_type',
            'target_type',
            'metadata_schema',
            'created_on',
            'modified_on',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = EntailmentType
        fields = (
            'source_type',
            'target_type',
            'metadata_schema',
        )


class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = EntailmentType
        fields = (
            'metadata_schema',
        )



from uuid import uuid4

from rest_framework.test import APITestCase
from irekua_database.utils import simple_JSON_schema
from irekua_rest_api.serializers import annotation_types
from .utils import (
    BaseTestCase,
    Users,
    Actions,
    create_permission_mapping_from_lists
)


class AnnotationTypeTestCase(BaseTestCase, APITestCase):
    serializer = annotation_types.CreateSerializer
    permissions = create_permission_mapping_from_lists({
        Actions.LIST: Users.ALL_AUTHENTICATED_USERS,
        Actions.CREATE: [
            Users.ADMIN,
            Users.DEVELOPER],
        Actions.RETRIEVE: Users.ALL_AUTHENTICATED_USERS,
        Actions.UPDATE: [
            Users.ADMIN,
            Users.DEVELOPER],
        Actions.PARTIAL_UPDATE: [
            Users.ADMIN,
            Users.DEVELOPER],
        Actions.DESTROY: [
            Users.ADMIN,
            Users.DEVELOPER],
    })

    @staticmethod
    def generate_random_json_data():
        data = {
            'name': str(uuid4()),
            'description': 'Random Annotation Tool',
            'annotation_schema': simple_JSON_schema()
        }
        return data



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from irekua_database.models import Site
from irekua_database.models import SamplingEvent
from irekua_database.models import SamplingEventDevice
from irekua_database.models import CollectionSite
from irekua_database.models import Item

from irekua_rest_api.serializers.base import IrekuaModelSerializer
from irekua_rest_api.serializers.base import IrekuaHyperlinkedModelSerializer
from irekua_rest_api.serializers.users import users


class SelectSerializer(IrekuaModelSerializer):
    class Meta:
        model = Site
        fields = (
            'url',
            'id',
        )


class ListSerializer(IrekuaModelSerializer):
    class Meta:
        model = Site
        fields = (
            'url',
            'id',
            'name',
            'locality',
        )


class DetailSerializer(IrekuaHyperlinkedModelSerializer):
    created_by = users.SelectSerializer(many=False, read_only=True)

    class Meta:
        model = Site
        fields = (
            'url',
            'id',
            'name',
            'locality',
            'created_by',
            'created_on',
            'modified_on',
        )


class FullDetailSerializer(IrekuaHyperlinkedModelSerializer):
    created_by = users.SelectSerializer(many=False, read_only=True)

    class Meta:
        model = Site
        fields = (
            'url',
            'id',
            'name',
            'locality',
            'latitude',
            'longitude',
            'geo_ref',
            'altitude',
            'created_by',
            'created_on',
            'modified_on',
        )


class CreateSerializer(IrekuaModelSerializer):
    class Meta:
        model = Site
        fields = (
            'name',
            'locality',
            'latitude',
            'longitude',
            'altitude',
        )

    def create(self, validated_data):
        user = self.context['request'].user
        validated_data['created_by'] = user
        return super().create(validated_data)


class UpdateSerializer(IrekuaModelSerializer):
    class Meta:
        model = Site
        fields = (
            'name',
            'locality',
            'altitude',
        )


class GeometrySerializer(IrekuaModelSerializer):
    class Meta:
        model = Site
        fields = (
            'latitude',
            'longitude',
            'altitude',
            'geo_ref',
        )


class SiteLocationSerializer(IrekuaModelSerializer):
    geometry = GeometrySerializer(read_only=True, source="*")

    class Meta:
        model = Site
        fields = (
            'id',
            'geometry'
        )

class SamplingEventLocationSerializer(IrekuaModelSerializer):
    geometry = GeometrySerializer(
        read_only=True,
        source="collection_site.site")

    class Meta:
        model = SamplingEvent
        fields = (
            'id',
            'geometry'
        )

class SamplingEventDeviceLocationSerializer(IrekuaModelSerializer):
    geometry = GeometrySerializer(
        read_only=True,
        source="sampling_event.collection_site.site")

    class Meta:
        model = SamplingEventDevice
        fields = (
            'id',
            'geometry'
        )


class ItemLocationSerializer(IrekuaModelSerializer):
    geometry = GeometrySerializer(
        read_only=True,
        source="sampling_event_device.sampling_event.collection_site.site")

    class Meta:
        model = Item
        fields = (
            'id',
            'geometry'
        )

class CollectionSiteLocationSerializer(IrekuaModelSerializer):
    geometry = GeometrySerializer(
        read_only=True,
        source="site")

    class Meta:
        model = CollectionSite
        fields = (
            'id',
            'geometry'
        )



from django import forms
import django_filters

from irekua_database.models import Term
from irekua_models.models import Model
from .utils import BaseFilter


search_fields = (
    'name',
    'annotation_type__name',
    'event_types__name',
    'terms__value',
)


class Filter(BaseFilter):
    terms = django_filters.ModelMultipleChoiceFilter(
        queryset=Term.objects.all(),
        widget=forms.TextInput)

    class Meta:
        model = Model
        fields = {
            'name': ['exact', 'icontains'],
            'annotation_type': ['exact'],
            'annotation_type__name': ['exact', 'icontains'],
            'event_types': ['exact'],
            'event_types__name': ['exact', 'icontains'],
            'terms__value': ['exact', 'icontains']
        }



from irekua_database.models import Term
from .utils import BaseFilter


search_fields = (
    'value',
)


class Filter(BaseFilter):
    class Meta:
        model = Term
        fields = (
            'value',
        )



from . import annotation_tools
from . import annotation_votes
from . import annotation_types
from . import annotations
from . import collection_types
from . import data_collections
from . import device_brands
from . import device_types
from . import devices
from . import entailment_types
from . import entailments
from . import event_types
from . import institutions
from . import item_types
from . import items
from . import licence_types
from . import metacollections
from . import physical_devices
from . import roles
from . import sampling_event_types
from . import sampling_events
from . import site_types
from . import sites
from . import synonym_suggestions
from . import synonyms
from . import tags
from . import term_suggestions
from . import term_types
from . import terms
from . import users
from . import collection_devices
from . import collection_users
from . import collection_sites
from . import licences
from . import sampling_event_devices
from . import secondary_items
from . import collection_administrators
from . import manager_collections
from . import models
from . import model_versions
from . import model_predictions

from .utils import BaseFilter


__all__ = [
    'BaseFilter',
    'annotation_tools',
    'annotation_types',
    'annotation_votes',
    'annotations',
    'collection_types',
    'data_collections',
    'device_brands',
    'device_types',
    'devices',
    'entailment_types',
    'entailments',
    'event_types',
    'institutions',
    'item_types',
    'items',
    'licence_types',
    'metacollections',
    'physical_devices',
    'roles',
    'sampling_event_types',
    'sampling_events',
    'site_types',
    'sites',
    'synonym_suggestions',
    'synonyms',
    'tags',
    'term_suggestions',
    'term_types',
    'terms',
    'users',
    'collection_users',
    'collection_sites',
    'collection_devices',
    'licences',
    'sampling_event_devices',
    'secondary_items',
    'collection_administrators',
    'manager_collections',
    'models',
    'model_versions',
    'model_predictions',
]



from irekua_database.models import Institution
from .utils import BaseFilter


search_fields = (
    'institution_name',
    'institution_code',
)


class Filter(BaseFilter):
    class Meta:
        model = Institution
        fields = (
            'institution_name',
            'institution_code',
            'subdependency',
            'country'
        )



from irekua_rest_api.apps import IrekuaRestApiConfig


def get_detail_view_name(model_class):
    return '{app_name}:{model_name}-detail'.format(
        app_name=IrekuaRestApiConfig.name,
        model_name=model_class._meta.object_name.lower())



from rest_framework import routers
from irekua_rest_api import views


main_router = routers.DefaultRouter()
main_router.register(
    r'annotations',
    views.AnnotationViewSet)
main_router.register(
    r'collections',
    views.CollectionViewSet)
main_router.register(
    r'devices',
    views.DeviceViewSet)
main_router.register(
    r'items',
    views.ItemViewSet)
main_router.register(
    r'sampling_events',
    views.SamplingEventViewSet)
main_router.register(
    r'sites',
    views.SiteViewSet)
main_router.register(
    r'terms',
    views.TermTypeViewSet)
main_router.register(
    r'users',
    views.UserViewSet)

main_router.register(
    r'types/mime_types',
    views.MimeTypeViewSet)
main_router.register(
    r'types/annotation_types',
    views.AnnotationTypeViewSet)
main_router.register(
    r'types/collection_types',
    views.CollectionTypeViewSet)
main_router.register(
    r'types/device_types',
    views.DeviceTypeViewSet)
main_router.register(
    r'types/entailment_types',
    views.EntailmentTypeViewSet)
main_router.register(
    r'types/event_types',
    views.EventTypeViewSet)
main_router.register(
    r'types/item_types',
    views.ItemTypeViewSet)
main_router.register(
    r'types/licence_types',
    views.LicenceTypeViewSet)
main_router.register(
    r'types/sampling_event_type_device_types',
    views.SamplingEventTypeDeviceTypeViewSet)
main_router.register(
    r'types/sampling_event_type_site_types',
    views.SamplingEventTypeSiteTypeViewSet)
main_router.register(
    r'types/sampling_event_types',
    views.SamplingEventTypeViewSet)
main_router.register(
    r'types/site_types',
    views.SiteTypeViewSet)
main_router.register(
    r'types/collection_type_site_types',
    views.CollectionTypeSiteTypeViewSet)
main_router.register(
    r'types/collection_type_administrators',
    views.CollectionTypeAdministratorViewSet)
main_router.register(
    r'types/collection_type_annotation_types',
    views.CollectionTypeAnnotationTypeViewSet)
main_router.register(
    r'types/collection_type_licence_types',
    views.CollectionTypeLicenceTypeViewSet)
main_router.register(
    r'types/collection_type_sampling_event_types',
    views.CollectionTypeSamplingEventTypeViewSet)
main_router.register(
    r'types/collection_type_item_types',
    views.CollectionTypeItemTypeViewSet)
main_router.register(
    r'types/collection_type_event_types',
    views.CollectionTypeEventTypeViewSet)
main_router.register(
    r'types/collection_type_device_types',
    views.CollectionTypeDeviceTypeViewSet)
main_router.register(
    r'types/collection_type_roles',
    views.CollectionTypeRoleViewSet)

main_router.register(
    r'additional/annotation_votes',
    views.AnnotationVoteViewSet)
main_router.register(
    r'additional/secondary_items',
    views.SecondaryItemViewSet)
main_router.register(
    r'additional/annotation_tools',
    views.AnnotationToolViewSet)
main_router.register(
    r'additional/collection_devices',
    views.CollectionDeviceViewSet)
main_router.register(
    r'additional/collection_sites',
    views.CollectionSiteViewSet)
main_router.register(
    r'additional/collection_users',
    views.CollectionUserViewSet)
main_router.register(
    r'additional/collection_administrators',
    views.CollectionAdministratorViewSet)
main_router.register(
    r'additional/device_brands',
    views.DeviceBrandViewSet)
main_router.register(
    r'additional/entailments',
    views.EntailmentViewSet)
main_router.register(
    r'additional/institutions',
    views.InstitutionViewSet)
main_router.register(
    r'additional/licences',
    views.LicenceViewSet)
main_router.register(
    r'additional/metacollections',
    views.MetaCollectionViewSet)
main_router.register(
    r'additional/physical_devices',
    views.PhysicalDeviceViewSet)
main_router.register(
    r'additional/roles',
    views.RoleViewSet)
main_router.register(
    r'additional/synonym_suggestions',
    views.SynonymSuggestionViewSet)
main_router.register(
    r'additional/synonyms',
    views.SynonymViewSet)
main_router.register(
    r'additional/tags',
    views.TagViewSet)
main_router.register(
    r'additional/term_suggestions',
    views.TermSuggestionViewSet)
main_router.register(
    r'additional/term_instances',
    views.TermViewSet)
main_router.register(
    r'additional/sampling_event_devices',
    views.SamplingEventDeviceViewSet)

main_router.register(
    r'models/models',
    views.ModelViewSet)
main_router.register(
    r'models/model_versions',
    views.ModelVersionViewSet)
main_router.register(
    r'models/model_predictions',
    views.ModelPredictionViewSet)
